@@ -18,21 +18,20 @@
 #
 """Artifacts defintions internally used to refer to intermediate results."""
 
-import os
 from enum import Enum
 from pathlib import Path
 
 from mlonmcu.setup import utils
 
-# class ModelLibraryFormatPlus:
-#     pass
-
 # TODO: offer pack/unpack/flatten methods for mlf
 # TODO: implement restore methods
 # TODO: decide if inheritance based scheme would fit better
+# TODO: add artifact flags and lookup utility to find best match
 
 
 class ArtifactFormat(Enum):  # TODO: ArtifactType, ArtifactKind?
+    """Enumeration of artifact types."""
+
     UNKNOWN = 0
     SOURCE = 1
     TEXT = 2
@@ -46,6 +45,29 @@ class ArtifactFormat(Enum):  # TODO: ArtifactType, ArtifactKind?
     PATH = 10  # NOT A DIRECTORY?
     RAW = 11
     BIN = 11
+    SHARED_OBJECT = 12  # Here: the parent tar archive
+
+
+def lookup_artifacts(artifacts, name=None, fmt=None, flags=None, first_only=False):
+    """Utility to get a matching artifact for a given set of properties."""
+    matches = []
+    # Warning: if neither name, fmt nor flags is provided, the first artifact (multiple=False)
+    # or all (multiple=True) are returned
+    for artifact in artifacts:
+        valid = True
+        if name is not None and artifact.name != name:
+            valid = False
+        if fmt is not None and artifact.fmt != fmt:
+            valid = False
+        if flags is not None and not all(
+            flag in artifact.flags for flag in flags
+        ):  # A valid artifact may have more flags than specified
+            valid = False
+        if valid:
+            matches.append(artifact)
+    if len(matches) > 0 and first_only:
+        matches = [matches[0]]
+    return matches
 
 
 class Artifact:
@@ -59,52 +81,70 @@ class Artifact:
         data=None,
         raw=None,
         fmt=ArtifactFormat.UNKNOWN,
+        flags=None,
         archive=False,
         optional=False,
     ):
         # TODO: Allow to store filenames as well as raw data
         self.name = name
+        # TODO: too many attributes...
         self.content = content
         self.path = path
         self.data = data
         self.raw = raw
         self.fmt = fmt
+        self.flags = flags if flags is not None else {}
         self.archive = archive
         self.optional = optional
         self.validate()
 
+    def __repr__(self):
+        return f"Artifact({self.name}, fmt={self.fmt}, flags={self.flags})"
+
     @property
     def exported(self):
+        """Returns true if the artifact was writtem to disk."""
         return bool(self.path is not None)
 
     def validate(self):
+        """Checker for artifact attributes for the given format."""
         if self.fmt in [ArtifactFormat.TEXT, ArtifactFormat.SOURCE]:
             assert self.content is not None
         elif self.fmt in [ArtifactFormat.RAW, ArtifactFormat.BIN]:
             assert self.raw is not None
-        elif self.fmt in [ArtifactFormat.MLF]:
-            assert self.raw is not None  # TODO: load it via tvm?
+        elif self.fmt in [ArtifactFormat.MLF, ArtifactFormat.SHARED_OBJECT]:
+            assert self.raw is not None
         elif self.fmt in [ArtifactFormat.PATH]:
             assert self.path is not None
         else:
             raise NotImplementedError
 
     def export(self, dest, extract=False):
+        """Export the artifact to a given path (file or directory) and update its path.
+
+        Arguments
+        ---------
+        dest : str
+            Path of the destination.
+        extract : bool
+            If archive: extract to destination.
+
+        """
         filename = Path(dest) / self.name
         if self.fmt in [ArtifactFormat.TEXT, ArtifactFormat.SOURCE]:
             assert not extract, "extract option is only available for ArtifactFormat.MLF"
-            with open(filename, "w") as handle:
+            with open(filename, "w", encoding="utf-8") as handle:
                 handle.write(self.content)
         elif self.fmt in [ArtifactFormat.RAW, ArtifactFormat.BIN]:
             assert not extract, "extract option is only available for ArtifactFormat.MLF"
             with open(filename, "wb") as handle:
                 handle.write(self.raw)
-        elif self.fmt in [ArtifactFormat.MLF]:
+        elif self.fmt in [ArtifactFormat.MLF, ArtifactFormat.SHARED_OBJECT]:
             with open(filename, "wb") as handle:
                 handle.write(self.raw)
             if extract:
                 utils.extract(filename, dest)
-                os.remove(filename)
+                # os.remove(filename)
         elif self.fmt in [ArtifactFormat.PATH]:
             assert not extract, "extract option is only available for ArtifactFormat.MLF"
             utils.copy(self.path, filename)
@@ -113,6 +153,7 @@ class Artifact:
         self.path = filename if self.path is None else self.path
 
     def print_summary(self):
+        """Utility to print information about an artifact to the cmdline."""
         print("Format:", self.fmt)
         print("Optional: ", self.optional)
         if self.fmt in [ArtifactFormat.TEXT, ArtifactFormat.SOURCE]:
@@ -120,7 +161,7 @@ class Artifact:
             print(self.content)
         elif self.fmt in [ArtifactFormat.RAW, ArtifactFormat.BIN]:
             print(f"Data Size: {len(self.raw)}B")
-        elif self.fmt in [ArtifactFormat.MLF]:
+        elif self.fmt in [ArtifactFormat.MLF, ArtifactFormat.SHARED_OBJECT]:
             print(f"Archive Size: {len(self.raw)}B")
         elif self.fmt in [ArtifactFormat.PATH]:
             print(f"File Location: {self.path}")
