@@ -17,9 +17,9 @@
 # limitations under the License.
 #
 import os
+import sys
 import multiprocessing
 import logging
-from mlonmcu.target import SUPPORTED_TARGETS
 
 from mlonmcu.platform import get_platforms
 from mlonmcu.session.postprocess import SUPPORTED_POSTPROCESSES
@@ -27,6 +27,8 @@ from mlonmcu.feature.features import get_available_feature_names
 from mlonmcu.logging import get_logger, set_log_level
 from .helper.parse import extract_config
 
+logger = get_logger()
+
 
 def handle_logging_flags(args):
     if hasattr(args, "verbose") and hasattr(args, "quiet"):
@@ -65,6 +67,7 @@ def add_flow_options(parser):
         metavar="PLATFORM",
         choices=get_platforms().keys(),
         default=None,
+        action="append",
         nargs=1,
         help="Explicitly choose the platforms to use (choices: %(choices)s)",
     )
@@ -125,7 +128,7 @@ def add_flow_options(parser):
         "--label",
         type=str,
         metavar="LABEL",
-        default="unamed",
+        default="",
         help="Label for the session (default: %(default)s)",
     )
 
@@ -178,14 +181,14 @@ def add_model_options(parser):
 def kickoff_runs(args, until, context):
     assert len(context.sessions) > 0
     session = context.sessions[-1]
-    session.label = args.label
+    # session.label = args.label
     config = extract_config(args)
     per_stage = True
     if "runs_per_stage" in config:
         per_stage = bool(config["runs_per_stage"])
     elif "runs_per_stage" in context.environment.vars:
         per_stage = bool(context.environment.vars["runs_per_stage"])
-    session.process_runs(
+    success = session.process_runs(
         until=until,
         per_stage=per_stage,
         num_workers=args.parallel,
@@ -193,3 +196,6 @@ def kickoff_runs(args, until, context):
         context=context,
         export=True,
     )
+    if not success:
+        logger.error("At least one error occured!")
+        sys.exit(1)
