@@ -18,15 +18,10 @@
 #
 """Command line subcommand for initializing a mlonmcu environment."""
 
-import sys
 import os
-import pkg_resources
-import pkgutil
-import jinja2
 
 from mlonmcu.environment.templates import get_template_names
-from mlonmcu.environment.config import get_environments_dir, DEFAULTS, env_subdirs
-from mlonmcu.utils import in_virtualenv
+from mlonmcu.environment.config import get_environments_dir, DEFAULTS
 from mlonmcu.environment.init import initialize_environment
 
 
@@ -47,9 +42,11 @@ def add_init_options(parser):
         metavar="TEMPLATE",
         nargs=1,
         type=str,
-        choices=get_template_names(),
+        # choices=get_template_names(),
         default=DEFAULTS["template"],
-        help="Environment template (default: %(default)s, allowed: %(choices)s)",
+        help="Environment template name or path (default: %(default)s, available: "
+        + ", ".join(get_template_names())
+        + ")",
     )
     init_parser.add_argument(
         "DIR",
@@ -77,13 +74,6 @@ def add_init_options(parser):
         action="store_true",
         help="Clone models directory into environment",
     )
-    init_parser.add_argument(
-        "--skip-sw",
-        dest="skip_sw",
-        default=None,
-        action="store_true",
-        help="Clone sw directory into environment",
-    )
     init_parser.add_argument(
         "--register",
         default=None,
@@ -116,7 +106,6 @@ def handle(args):
         create_venv=args.venv,
         interactive=not args.non_interactive,
         clone_models=args.clone_models,
-        skip_sw=args.skip_sw,
         register=args.register,
         template=args.template[0] if isinstance(args.template, list) else args.template,
         allow_exists=args.allow_exists,
