@@ -182,7 +182,7 @@ def load_recent_sessions(env: Environment, count: int = None) -> List[Session]:
         runs = []
         for rid in run_ids:
             run_directory = runs_directory / str(rid)
-            run_file = run_directory / "run.txt"
+            # run_file = run_directory / "run.txt"
             # run = Run.from_file(run_file)  # TODO: actually implement run restore
             run = Run()  # TODO: fix
             run.archived = True
@@ -277,18 +277,22 @@ class MlonMcuContext:
         self.lock = lock
         self.lockfile = filelock.FileLock(os.path.join(self.environment.home, ".lock"))
         self.sessions = load_recent_sessions(self.environment)
+        if self.environment.defaults.cleanup_auto:
+            logger.debug("Cleaning up old sessions automaticaly")
+            self.cleanup_sessions(keep=self.environment.defaults.cleanup_keep, interactive=False)
+            self.sessions = load_recent_sessions(self.environment)
         self.session_idx = self.sessions[-1].idx if len(self.sessions) > 0 else -1
         logger.debug(f"Restored {len(self.sessions)} recent sessions")
         self.cache = TaskCache()
 
-    def create_session(self):
+    def create_session(self, label="", config=None):
         """Create a new session in the current context."""
         idx = self.session_idx + 1
         logger.debug("Creating a new session with idx %s", idx)
         temp_directory = self.environment.paths["temp"].path
         sessions_directory = temp_directory / "sessions"
         session_dir = sessions_directory / str(idx)
-        session = Session(idx=idx, dir=session_dir)
+        session = Session(idx=idx, label=label, dir=session_dir, config=config)
         self.sessions.append(session)
         self.session_idx = idx
         # TODO: move this to a helper function
@@ -313,7 +317,7 @@ class MlonMcuContext:
                             return
         logger.info("No cache found in deps directory")
 
-    def get_session(self, resume=False) -> Session:
+    def get_session(self, label="", resume=False, config=None) -> Session:
         """Get an active session if available, else create a new one.
 
         Returns
@@ -323,12 +327,12 @@ class MlonMcuContext:
         """
         if resume:
             assert len(self.sessions) > 0, "There is no recent session available"
-            session = self.sessions[-1]
+            # session = self.sessions[-1]
             assert False, "The latest session can not be resumed"
             raise NotImplementedError
 
         if self.session_idx < 0 or not self.sessions[-1].active:
-            self.create_session()
+            self.create_session(label=label, config=config)
         return self.sessions[-1]
 
     def __enter__(self):
@@ -361,15 +365,17 @@ class MlonMcuContext:
         """Utility to cleanup old sessions from the disk."""
         assert self.is_clean
         all_sessions = self.sessions
-        # print("all_sessions", all_sessions)
         to_keep = all_sessions[-keep:] if keep > 0 else []
         to_remove = self.sessions[:-keep] if keep > 0 else self.sessions
         count = len(to_remove)
         if count > 0:
             temp_dir = self.environment.lookup_path("temp").path
             sessions_dir = temp_dir / "sessions"
-            print(f"The following {count} sessions will be removed from the environments temp directory ({temp_dir}):")
-            print(" ".join([str(session.idx) for session in to_remove]))
+            if interactive:
+                print(
+                    f"The following {count} sessions will be removed from the environments temp directory ({temp_dir}):"
+                )
+                print(" ".join([str(session.idx) for session in to_remove]))
 
             if ask_user("Are your sure?", default=not interactive, interactive=interactive):
                 for session in to_remove:
@@ -380,12 +386,39 @@ class MlonMcuContext:
                     shutil.rmtree(session_dir)
                 self.sessions = to_keep
                 self.session_idx = self.sessions[-1].idx if len(self.sessions) > 0 else -1
-                print("Done")
+                if interactive:
+                    print("Done")
             else:
-                print("Aborted")
+                if interactive:
+                    print("Aborted")
         else:
-            print("No sessions selected for removal")
-        # We currently do not support rewirting the indices to start from scratch again as this would lead to inconsitencies with the path in the report/cmake build dirtectory
+            if interactive:
+                print("No sessions selected for removal")
+        # We currently do not support rewirting the indices to start from scratch again as this
+        # would lead to inconsitencies with the path in the report/cmake build dirtectory
+
+    def get_sessions_runs_idx(self):
+        sessions_dict = {}
+        for session in self.sessions:
+            session_runs_idx = [run.idx if run.idx is not None else i for i, run in enumerate(session.runs)]
+            sessions_dict[session.idx] = session_runs_idx
+        return sessions_dict
+
+    def print_summary(self, sessions=True, runs=False, labels=True):
+        def print_sessions(sessions_runs, with_runs=False, with_labels=True):
+            print("Sessions:")
+            session_index_map = {session.idx: i for i, session in enumerate(self.sessions)}
+            for session, runs in sessions_runs.items():
+                print(f"  - {session}", end="")
+                if with_runs:
+                    print(" (" + self.sessions[session_index_map[session]].label + ")", end="")
+                if with_runs:
+                    print(" [runs: " + " ".join([str(run) for run in runs]) + "]", end="")
+                print()
+
+        print("Context Summary\n")
+        sessions_runs = self.get_sessions_runs_idx()
+        print_sessions(sessions_runs, with_runs=runs, with_labels=labels)
 
     def export(self, dest, session_ids=None, run_ids=None, interactive=True):
         dest = Path(dest)
