@@ -16,11 +16,23 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-from pathlib import Path
 import logging
-import yaml
 
-from .config import *
+from .config import (
+    DefaultsConfig,
+    PathConfig,
+    RepoConfig,
+    FrameworkConfig,
+    FrameworkFeatureConfig,
+    BackendConfig,
+    BackendFeatureConfig,
+    TargetConfig,
+    TargetFeatureConfig,
+    PlatformConfig,
+    PlatformFeatureConfig,
+    FrontendConfig,
+    FrontendFeatureConfig,
+)
 from .loader import load_environment_from_file
 from .writer import write_environment_to_file
 
@@ -280,7 +292,9 @@ class Environment:
         configs = self.lookup_target_configs(target=name)
         return len(configs) > 0
 
-    # def has_postprocess(self, name):  # TODO: actually we do not need to explicitly enable those? environment.yml list the default enabled ones instead of the supported ones in the environment
+    # TODO: actually we do not need to explicitly enable those? environment.yml list the default enabled ones instead
+    # of the supported ones in the environment
+    # def has_postprocess(self, name):
     #     configs = self.lookup_postprocess_configs(postprocess=name)
     #     return len(configs) > 0
 
@@ -288,8 +302,8 @@ class Environment:
         if framework is None or framework not in self.defaults.default_backends:
             return []
         default = self.defaults.default_backends[framework]
-        framework_names = [framework_config.name for framework_config in self.frameworks]
-        framework_config = self.frameworks[framework_names.index(framework)]
+        # framework_names = [framework_config.name for framework_config in self.frameworks]
+        # framework_config = self.frameworks[framework_names.index(framework)]
         if default is None:
             return []
         if isinstance(default, str):
@@ -336,6 +350,8 @@ class DefaultEnvironment(Environment):
             default_framework=None,
             default_backends={},
             default_target=None,
+            cleanup_auto=False,
+            cleanup_keep=100,
         )
         self.paths = {
             "deps": PathConfig("./deps"),
