@@ -19,7 +19,6 @@
 """ MLonMCU Features API"""
 
 from abc import ABC
-from typing import List
 
 from mlonmcu.config import filter_config
 from .type import FeatureType
@@ -32,14 +31,17 @@ class FeatureBase(ABC):
     """Feature base class"""
 
     feature_type = None
+    scope = None
 
     DEFAULTS = {"enabled": True}
     REQUIRED = []
 
-    def __init__(self, name, config=None):
+    def __init__(self, name, features=None, config=None):
         self.name = name
         self.config = config if config else {}
         self.config = filter_config(self.config, self.name, self.DEFAULTS, self.REQUIRED)
+        # assert features is None, "Features with features are currently not supported"
+        # just ignore the supplied feature list for now
 
     @property
     def enabled(self):
@@ -86,8 +88,8 @@ class FrontendFeature(FeatureBase):
 
     feature_type = FeatureType.FRONTEND
 
-    def __init__(self, name, config=None):
-        super().__init__(name, config=config)
+    def __init__(self, name, features=None, config=None):
+        super().__init__(name, features=features, config=config)
 
     def get_frontend_config(self, frontend):
         # pylint: disable=unused-argument
@@ -96,14 +98,17 @@ class FrontendFeature(FeatureBase):
     def add_frontend_config(self, frontend, config):
         config.update(self.get_frontend_config(frontend))
 
+    def update_formats(self, frontend, input_formats, output_formats):
+        pass
+
 
 class FrameworkFeature(FeatureBase):
     """Framework related feature"""
 
     feature_type = FeatureType.FRAMEWORK
 
-    def __init__(self, name, config=None):
-        super().__init__(name, config=config)
+    def __init__(self, name, features=None, config=None):
+        super().__init__(name, features=features, config=config)
 
     def get_framework_config(self, framework):
         # pylint: disable=unused-argument
@@ -118,8 +123,8 @@ class BackendFeature(FeatureBase):
 
     feature_type = FeatureType.BACKEND
 
-    def __init__(self, name, config=None):
-        super().__init__(name, config=config)
+    def __init__(self, name, features=None, config=None):
+        super().__init__(name, features=features, config=config)
 
     def get_backend_config(self, backend):
         # pylint: disable=unused-argument
@@ -135,8 +140,8 @@ class TargetFeature(FeatureBase):
 
     feature_type = FeatureType.TARGET
 
-    def __init__(self, name, config=None):
-        super().__init__(name, config=config)
+    def __init__(self, name, features=None, config=None):
+        super().__init__(name, features=features, config=config)
 
     def get_target_config(self, target):
         # pylint: disable=unused-argument
@@ -146,14 +151,23 @@ class TargetFeature(FeatureBase):
         # TODO: cfg passed to method instead of contructor or self.config = config
         config.update(self.get_target_config(target))
 
+    def get_target_callback(self, target):
+        return None
+
+    def add_target_callback(self, target, callbacks):
+        # TODO: cfg passed to method instead of contructor or self.config = config
+        callback = self.get_target_callback(target)
+        if callback is not None:
+            callbacks.append(callback)
+
 
 class PlatformFeature(FeatureBase):
     """Platform/Compile related feature"""
 
     feature_type = FeatureType.PLATFORM
 
-    def __init__(self, name, config=None):
-        super().__init__(name, config=config)
+    def __init__(self, name, features=None, config=None):
+        super().__init__(name, features=features, config=config)
 
     def get_platform_config(self, platform):
         return {}
@@ -173,8 +187,8 @@ class SetupFeature(FeatureBase):  # TODO: alternative: CacheFeature
 
     feature_type = FeatureType.SETUP
 
-    def __init__(self, name, config=None):
-        super().__init__(name, config=config)
+    def __init__(self, name, features=None, config=None):
+        super().__init__(name, features=features, config=config)
 
     def get_setup_config(self):
         raise NotImplementedError
@@ -201,8 +215,8 @@ class RunFeature(FeatureBase):
 
     feature_type = FeatureType.RUN
 
-    def __init__(self, name, config=None):
-        super().__init__(name, config=config)
+    def __init__(self, name, features=None, config=None):
+        super().__init__(name, features=features, config=config)
 
     def get_run_config(self):
         return {}
