@@ -18,7 +18,6 @@
 #
 import os
 import argparse
-import logging
 from pathlib import Path
 from abc import ABC, abstractmethod
 
@@ -26,7 +25,7 @@ from mlonmcu.cli.helper.parse import extract_feature_names, extract_config
 from mlonmcu.feature.type import FeatureType
 from mlonmcu.config import filter_config
 from mlonmcu.feature.features import get_matching_features
-from mlonmcu.artifact import Artifact
+from mlonmcu.artifact import ArtifactFormat
 from mlonmcu.logging import get_logger
 
 logger = get_logger()
@@ -53,6 +52,7 @@ class Backend(ABC):
         self.config = filter_config(self.config, self.name, self.DEFAULTS, self.REQUIRED)
         self.context = context
         self.artifacts = []
+        self.supported_fmts = []
         self.tuner = None
 
     def __repr__(self):
@@ -74,7 +74,7 @@ class Backend(ABC):
         pass
 
     @abstractmethod
-    def generate_code(self, verbose=False):
+    def generate_code(self):
         pass
 
     @property
@@ -94,7 +94,7 @@ class Backend(ABC):
     def export_tuning_results(self, path):
         if not self.has_tuner:
             raise NotImplementedError("Backend does not support autotuning")
-        artifact = self.tuner.get_results()
+        # artifact = self.tuner.get_results()
         # TODO: write tuning report to file...
         raise NotImplementedError
 
@@ -110,7 +110,7 @@ class Backend(ABC):
                 path.is_dir()
             ), "The supplied path does not exists."  # Make sure it actually exists (we do not create it by default)
             for artifact in self.artifacts:
-                extract = artifact.fmt == ArctifactFormat.MLF
+                extract = artifact.fmt == ArtifactFormat.MLF
                 artifact.export(path, extract=extract)
                 # TODO: move the following to a helper function and share code
                 # dest = path / artifact.name
@@ -146,6 +146,9 @@ class Backend(ABC):
     def add_platform_defs(self, platform, defs):
         defs.update(self.get_platform_defs(platform))
 
+    def supports_model(self, model):
+        return any(fmt in self.supported_formats for fmt in model.formats)
+
 
 def get_parser(backend_name, features, required, defaults):
     # TODO: add help strings should start with a lower case letter
@@ -191,7 +194,7 @@ def get_parser(backend_name, features, required, defaults):
         metavar="KEY=VALUE",
         nargs="+",
         action="append",
-        help=f"""Set a number of key-value pairs
+        help="""Set a number of key-value pairs
 
 Allowed options:
 """
@@ -204,12 +207,13 @@ Allowed options:
 
 
 def init_backend_features(names, config):
-    features = []
-    for name in names:
-        feature_classes = get_supported_features(feature_type=FeatureType.BACKEND, feature_name=name)
-        for feature_class in feature_classes:
-            features.append(feature_class(config=config))
-    return features
+    raise NotImplementedError
+    # features = []
+    # for name in names:
+    #     feature_classes = get_supported_features(feature_type=FeatureType.BACKEND, feature_name=name)
+    #     for feature_class in feature_classes:
+    #         features.append(feature_class(config=config))
+    # return features
 
 
 def main(backend, args=None):
@@ -230,7 +234,9 @@ def main(backend, args=None):
     features = init_backend_features(features_names, config)
     backend_inst = backend(features=features, config=config)
     backend_inst.load_model(model)
-    backend_inst.generate_code(verbose=args.verbose)
+    if args.verbose:
+        config["print_outputs"] = True
+    backend_inst.generate_code()
     if args.print:
         print("Printing generated artifacts:")
         for artifact in backend_inst.artifacts:
