@@ -16,7 +16,7 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-from abc import ABC, abstractmethod
+from abc import ABC
 
 from mlonmcu.feature.type import FeatureType
 from mlonmcu.config import filter_config
