@@ -18,4 +18,6 @@
 #
 """TVM framework module."""
 
-from .backend import *
+from .backend import TVMAOTBackend, TVMRTBackend, TVMCGBackend
+
+__all__ = ["TVMAOTBackend", "TVMRTBackend", "TVMCGBackend"]
