@@ -25,7 +25,6 @@ import tarfile
 
 from .backend import TVMBackend
 from .tvmrt import TVMRTBackend
-from .wrapper import generate_wrapper_header
 from mlonmcu.flow.backend import main
 from mlonmcu.artifact import Artifact, ArtifactFormat
 from mlonmcu.setup import utils
@@ -35,6 +34,10 @@ class TVMCGBackend(TVMRTBackend):
 
     name = "tvmcg"
 
+    FEATURES = [
+        *TVMBackend.FEATURES,
+    ]
+
     REQUIRED = TVMRTBackend.REQUIRED + ["utvmcg.exe"]
 
     def get_max_workspace_size_from_metadata(self, metadata):
@@ -46,7 +49,7 @@ class TVMCGBackend(TVMRTBackend):
             )
         return max_workspace
 
-    def generate_code(self, verbose=False):
+    def generate_code(self):
         super().generate_code()
         artifacts = self.artifacts
         artifact = None
@@ -70,9 +73,15 @@ class TVMCGBackend(TVMRTBackend):
                     args.append(params_bin_file)
                     args.append(out_file)
                     args.append(str(max_workspace_size))
-                    utils.exec_getout(tvmcg_exe, *args, live=verbose, print_output=False)
+                    out = utils.exec_getout(tvmcg_exe, *args, live=self.print_outputs, print_output=False)
                     codegen_src = open(out_file, "r").read()
                     artifact = Artifact("staticrt.c", content=codegen_src, fmt=ArtifactFormat.SOURCE)
+                    workspace_size_artifact = Artifact(
+                        "tvmcg_workspace_size.txt", content=f"{max_workspace_size}", fmt=ArtifactFormat.TEXT
+                    )
+                    artifacts.append(workspace_size_artifact)
+                    stdout_artifact = Artifact("tvmcg_out.log", content=out, fmt=ArtifactFormat.TEXT)
+                    artifacts.append(stdout_artifact)
                 break
         assert artifact is not None, "Failed to find MLF artifact"
         artifacts.append(artifact)
