@@ -18,7 +18,8 @@
 #
 import sys
 import tempfile
-import json
+
+# import json
 import tarfile
 from pathlib import Path
 
@@ -26,6 +27,7 @@ from .backend import TVMBackend
 from .wrapper import generate_tvmrt_wrapper, generate_wrapper_header
 from mlonmcu.flow.backend import main
 from mlonmcu.artifact import Artifact, ArtifactFormat
+from .tvmc_utils import get_tvmrt_tvmc_args
 
 
 class TVMRTBackend(TVMBackend):
@@ -37,7 +39,8 @@ class TVMRTBackend(TVMBackend):
 
     DEFAULTS = {
         **TVMBackend.DEFAULTS,
-        "arena_size": 2 ** 20,  # Can not be detemined automatically (Very large)
+        "debug_arena": False,
+        "arena_size": 2**20,  # Can not be detemined automatically (Very large)
         # TODO: arena size warning!
     }
 
@@ -48,13 +51,12 @@ class TVMRTBackend(TVMBackend):
         size = self.config["arena_size"]
         return int(size) if size else None
 
-    def get_tvmc_compile_args(self):
-        return super().get_tvmc_compile_args("graph") + [
-            "--runtime-crt-system-lib",
-            str(1),
-            "--executor-graph-link-params",
-            str(0),
-        ]
+    @property
+    def debug_arena(self):
+        return bool(self.config["debug_arena"])
+
+    def get_tvmc_compile_args(self, out):
+        return super().get_tvmc_compile_args(out, executor="graph") + get_tvmrt_tvmc_args()
 
     def get_graph_and_params_from_mlf(self, path):
         graph = None
@@ -66,19 +68,19 @@ class TVMRTBackend(TVMBackend):
 
         return graph, params
 
-    def generate_code(self, verbose=False):
+    def generate_code(self):
         artifacts = []
         assert self.model is not None
         full = False  # Required due to bug in TVM
         dump = ["c", "relay"] if full else []
         with tempfile.TemporaryDirectory() as temp_dir:
             out_path = Path(temp_dir) / f"{self.prefix}.tar"
-            self.invoke_tvmc_compile(out_path, dump=dump, verbose=verbose)
+            out = self.invoke_tvmc_compile(out_path, dump=dump)
             mlf_path = Path(temp_dir) / "mlf"
             tarfile.open(out_path).extractall(mlf_path)
-            with open(mlf_path / "metadata.json") as handle:
-                metadata = json.load(handle)
-            metadata_txt = json.dumps(metadata)
+            # with open(mlf_path / "metadata.json") as handle:
+            #     metadata = json.load(handle)
+            # metadata_txt = json.dumps(metadata)
             with open(out_path, "rb") as handle:
                 mlf_data = handle.read()
                 artifacts.append(
@@ -93,9 +95,9 @@ class TVMRTBackend(TVMBackend):
                 with open(str(out_path) + ".c", "r") as handle:
                     mod_src = handle.read()
                     artifacts.append(
-                        Artifac(
+                        Artifact(
                             f"{self.prefix}.c",
-                            content=mod_str,
+                            content=mod_src,
                             fmt=ArtifactFormat.SOURCE,
                             optional=True,
                         )
@@ -103,7 +105,7 @@ class TVMRTBackend(TVMBackend):
                 with open(str(out_path) + ".relay", "r") as handle:
                     mod_txt = handle.read()
                     artifacts.append(
-                        Artifac(
+                        Artifact(
                             f"{self.prefix}.relay",
                             content=mod_txt,
                             fmt=ArtifactFormat.TEXT,
@@ -115,10 +117,20 @@ class TVMRTBackend(TVMBackend):
                 workspace_size = self.arena_size
                 assert workspace_size >= 0
                 graph, params = self.get_graph_and_params_from_mlf(mlf_path)
-                wrapper_src = generate_tvmrt_wrapper(graph, params, self.model_info, workspace_size)
+                wrapper_src = generate_tvmrt_wrapper(
+                    graph, params, self.model_info, workspace_size, debug_arena=self.debug_arena
+                )
                 artifacts.append(Artifact("rt_wrapper.c", content=wrapper_src, fmt=ArtifactFormat.SOURCE))
                 header_src = generate_wrapper_header()
                 artifacts.append(Artifact("tvm_wrapper.h", content=header_src, fmt=ArtifactFormat.SOURCE))
+            workspace_size_artifact = Artifact(
+                "tvmrt_workspace_size.txt", content=f"{workspace_size}", fmt=ArtifactFormat.TEXT
+            )
+            artifacts.append(workspace_size_artifact)
+            stdout_artifact = Artifact(
+                "tvmc_compile_out.log", content=out, fmt=ArtifactFormat.TEXT
+            )  # TODO: rename to tvmrt_out.log?
+            artifacts.append(stdout_artifact)
 
         # prepare -> common?
         # invoke_tvmc -> common?
