@@ -21,10 +21,7 @@ import os
 import glob
 from itertools import product
 import yaml
-import logging
-from enum import Enum
 
-from .metadata import parse_metadata
 from .model import Model, ModelFormats
 from .group import ModelGroup
 
@@ -53,13 +50,13 @@ def find_metadata(directory, model_name=None):
         filename = f"{combination[0]}.{combination[1]}"
         fullpath = directory / filename
         if fullpath.is_file():
+            return fullpath
             # logger.debug("Found match. Ignoring other files")
-            metadata = parse_metadata(fullpath)
-            return metadata
     return None
 
 
-def list_models(directory, depth=1, formats=None):
+def list_models(directory, depth=1, formats=None, config=None):  # TODO: get config from environment or cmdline!
+    config = config if config is not None else {}
     formats = formats if formats else [ModelFormats.TFLITE]
     assert len(formats) > 0, "No formats peovided for model lookup"
     for fmt in formats:
@@ -97,27 +94,35 @@ def list_models(directory, depth=1, formats=None):
                     submodels.remove(main_model)
 
                     main_base = main_model.split("/")[-1]
-                    main_metadata = find_metadata(Path(directory) / dirname, model_name=main_base)
+                    main_config = {}
+                    main_metadata_path = find_metadata(Path(directory) / dirname, model_name=main_base)
+                    if main_metadata_path:
+                        main_config[f"{main_base}.metadata_path"] = main_metadata_path
+                    main_config.update(config)
 
                     models.append(
                         Model(
                             main_base,
                             [Path(directory) / f"{main_model}.{ext}"],
+                            config=main_config,
                             alt=main_model,
-                            formats=[ModelFormats.TFLITE],
-                            metadata=main_metadata,
+                            formats=[fmt],
                         )
                     )
 
                 for submodel in submodels:
                     sub_base = submodel.split("/")[-1]
-                    submodel_metadata = find_metadata(Path(directory) / dirname, model_name=sub_base)
+                    submodel_config = {}
+                    submodel_metadata_path = find_metadata(Path(directory) / dirname, model_name=sub_base)
+                    if submodel_metadata_path:
+                        submodel_config[f"{sub_base}.metadata_path"] = submodel_metadata_path
+                    submodel_config.update(config)
                     models.append(
                         Model(
                             submodel,
                             [Path(directory) / f"{submodel}.{ext}"],
-                            formats=[ModelFormats.TFLITE],
-                            metadata=submodel_metadata,
+                            config=submodel_config,
+                            formats=[fmt],
                         )
                     )
 
@@ -256,22 +261,10 @@ def print_summary(context, detailed=False):
     print_groups(groups, duplicates=group_duplicates, all_models=models, detailed=detailed)
 
 
-def unpack_modelgroups(names, groups):
-    ret = []
-    group_names = [group.name for group in groups]
-    for name in names:
-        if name in group_names:
-            index = group_names.index(name)
-            ret.extend(groups[index].models)
-        else:
-            ret.append(name)
-    return list(dict.fromkeys(ret))  # Drop duplicates
-
-
 def lookup_models(names, frontends=None, context=None):
     if frontends is None:
         assert context is not None
-        # TODO: Get defaults backends from environment (with no config/features)
+        # TODO: Get defaults frontends from environment (with no config/features)
         raise NotImplementedError
         # frontends = ?
     # allowed_ext = [frontend.fmt.extension for frontend in frontends]
@@ -280,8 +273,7 @@ def lookup_models(names, frontends=None, context=None):
 
     if context:
         directories = get_model_directories(context)
-        models, groups, _, _ = lookup_models_and_groups(directories, allowed_fmts)
-        names = unpack_modelgroups(names, groups)
+        models, _, _, _ = lookup_models_and_groups(directories, allowed_fmts)
     else:
         models = []
     model_names = [model.name for model in models]
@@ -289,13 +281,18 @@ def lookup_models(names, frontends=None, context=None):
     hints = []
     for name in names:
         filepath = Path(name)
-        ext = filepath.suffix
+        real_name = filepath.stem
+        ext = filepath.suffix[1:]
         if len(ext) > 0 and filepath.is_file():  # Explicit file
             assert (
                 ext in allowed_exts
             ), f"Unsupported file extension for model which was explicitly passed by path: {ext}"
             paths = [filepath]
-            hint = Model(name, paths, format=ModelFormat.from_extension(ext))
+            config = {}
+            metadata_path = find_metadata(filepath.parent.resolve(), model_name=real_name)
+            if metadata_path:
+                config[f"{real_name}.metadata_path"] = metadata_path
+            hint = Model(real_name, paths, config=config, formats=[ModelFormats.from_extension(ext)])
             # TODO: look for metadata
             hints.append(hint)
         else:
@@ -309,6 +306,19 @@ def lookup_models(names, frontends=None, context=None):
     return hints
 
 
+def apply_modelgroups(models, context=None):
+    assert context is not None
+    directories = get_model_directories(context)
+
+    for directory in directories:
+        groups = list_modelgroups(directory)
+        for i, group in enumerate(groups):
+            if group.name in models:
+                models = [*models[:i], *group.models, *models[i + 1 :]]
+                break
+    return list(dict.fromkeys(models))  # Drop duplicates
+
+
 def map_frontend_to_model(model, frontends, backend=None):
     model_fmts = model.formats
     backend_fmts = backend.get_supported_formats() if backend else []
