@@ -27,19 +27,34 @@ from mlonmcu.logging import get_logger
 from mlonmcu.target import SUPPORTED_TARGETS
 from mlonmcu.target.target import Target
 
-from .platform import CompilePlatform
+from .platform import CompilePlatform, TargetPlatform
 from .mlif_target import get_mlif_targets, create_mlif_target
 
 logger = get_logger()
 
 
-class MlifPlatform(CompilePlatform):
+class MlifPlatform(CompilePlatform, TargetPlatform):
     """Model Library Interface Platform class."""
 
-    FEATURES = CompilePlatform.FEATURES + ["validate"]
+    FEATURES = (
+        CompilePlatform.FEATURES
+        + TargetPlatform.FEATURES
+        + [
+            "validate",
+            "muriscvnn",
+            "cmsisnn",
+            "muriscvnnbyoc",
+            "cmsisnnbyoc",
+            "vext",
+            "pext",
+            "arm_mvei",
+            "arm_dsp",
+        ]  # TODO: allow Feature-Features with automatic resolution of initialization order
+    )
 
     DEFAULTS = {
         **CompilePlatform.DEFAULTS,
+        **TargetPlatform.DEFAULTS,
         "ignore_data": True,
         "fail_on_error": False,  # Prefer to add acolum with validation results instead of raising a RuntimeError
         "model_support_dir": None,
@@ -49,16 +64,11 @@ class MlifPlatform(CompilePlatform):
 
     REQUIRED = ["mlif.src_dir"]
 
-    # def __init__(self, framework, backend, target, features=None, config=None, context=None):
     def __init__(self, features=None, config=None):
         super().__init__(
             "mlif",
-            # framework=framework,
-            # backend=backend,
-            # target=target,
             features=features,
             config=config,
-            # context=context,
         )
         self.tempdir = None
         self.build_dir = None
@@ -144,6 +154,7 @@ class MlifPlatform(CompilePlatform):
         self.init_directory()
 
     def configure(self, target, src, _model, num=1, data_file=None):
+        del target
         if not isinstance(src, Path):
             src = Path(src)
         cmakeArgs = []
@@ -170,31 +181,42 @@ class MlifPlatform(CompilePlatform):
             assert data_file is not None, "No data.c file was supplied"
             cmakeArgs.append("-DDATA_SRC=" + str(data_file))
         utils.mkdirs(self.build_dir)
-        utils.cmake(
+        out = utils.cmake(
             self.mlif_dir,
             *cmakeArgs,
             cwd=self.build_dir,
             debug=self.debug,
-            live=self.print_output,
+            live=self.print_outputs,
         )
+        return out
 
     def compile(self, target, src=None, model=None, num=1, data_file=None):
+        out = ""
         if src:
-            self.configure(target, src, model, num=num, data_file=data_file)
-        utils.make(
+            out += self.configure(target, src, model, num=num, data_file=data_file)
+        out += utils.make(
             self.goal,
             cwd=self.build_dir,
             threads=self.num_threads,
-            live=self.print_output,
+            live=self.print_outputs,
         )
+        return out
 
-    def generate_elf(self, target, src=None, model=None, num=1, data_file=None):
+    def generate_elf(self, src, target, model=None, num=1, data_file=None):
         artifacts = []
-        self.compile(target, src=src, model=model, num=num, data_file=data_file)
+        out = self.compile(target, src=src, model=model, num=num, data_file=data_file)
         elf_file = self.build_dir / "bin" / "generic_mlif"
         # TODO: just use path instead of raw data?
         with open(elf_file, "rb") as handle:
             data = handle.read()
             artifact = Artifact("generic_mlif", raw=data, fmt=ArtifactFormat.RAW)
             artifacts.append(artifact)
+        metrics = self.get_metrics(elf_file)
+        content = metrics.to_csv(include_optional=True)  # TODO: store df instead?
+        metrics_artifact = Artifact("metrics.csv", content=content, fmt=ArtifactFormat.TEXT)
+        artifacts.append(metrics_artifact)
+        stdout_artifact = Artifact(
+            "mlif_out.log", content=out, fmt=ArtifactFormat.TEXT
+        )  # TODO: rename to tvmaot_out.log?
+        artifacts.append(stdout_artifact)
         self.artifacts = artifacts
