@@ -18,14 +18,16 @@
 #
 import tempfile
 import multiprocessing
-import distutils.util
 from pathlib import Path
 from filelock import FileLock
 
 from mlonmcu.config import filter_config
 from mlonmcu.feature.features import get_matching_features
 from mlonmcu.feature.type import FeatureType
+from mlonmcu.target.metrics import Metrics
+from mlonmcu.target.elf import get_results as get_static_mem_usage
 from mlonmcu.logging import get_logger
+from mlonmcu.config import str2bool
 
 logger = get_logger()
 
@@ -50,7 +52,9 @@ class Platform:
 
     FEATURES = []
 
-    DEFAULTS = {}
+    DEFAULTS = {
+        "print_outputs": False,
+    }
 
     REQUIRED = []
 
@@ -61,6 +65,7 @@ class Platform:
         # self.backend = backend
         # self.target = target
         self.config = config if config else {}
+        self.definitions = {}
         self.features = self.process_features(features)
         self.config = filter_config(self.config, self.name, self.DEFAULTS, self.REQUIRED)
         # self.context = context
@@ -69,10 +74,6 @@ class Platform:
     def init_directory(self, path=None, context=None):
         raise NotImplementedError
 
-    @staticmethod
-    def create_target(name, features=None, config=None):
-        raise NotImplementedError
-
     @property
     def supports_compile(self):
         return False
@@ -85,6 +86,10 @@ class Platform:
     def supports_monitor(self):
         return False
 
+    @property
+    def print_outputs(self):
+        return str2bool(self.config["print_outputs"])
+
     def process_features(self, features):
         if features is None:
             return []
@@ -92,10 +97,11 @@ class Platform:
         for feature in features:
             assert feature.name in self.FEATURES, f"Incompatible feature: {feature.name}"
             feature.add_platform_config(self.name, self.config)
+            feature.add_platform_defs(self.name, self.definitions)
         return features
 
     def get_supported_targets(self):
-        return NotImplementedError
+        return []
 
 
 class CompilePlatform(Platform):
@@ -105,7 +111,6 @@ class CompilePlatform(Platform):
 
     DEFAULTS = {
         **Platform.DEFAULTS,
-        "print_output": False,
         "debug": False,
         "build_dir": None,
         "num_threads": multiprocessing.cpu_count(),
@@ -113,26 +118,16 @@ class CompilePlatform(Platform):
 
     REQUIRED = []
 
-    # def __init__(self, name, framework, backend, target, features=None, config=None, context=None):
     def __init__(self, name, features=None, config=None):
         super().__init__(
             name,
-            # framework,
-            # backend,
-            # target,
             features=features,
             config=config,
-            # context=context,
         )
         self.name = name
-        # self.framework = framework  # TODO: required? or self.target.framework?
-        # self.backend = backend
-        # self.target = target
         self.config = config if config else {}
         self.features = self.process_features(features)
         self.config = filter_config(self.config, self.name, self.DEFAULTS, self.REQUIRED)
-        self.definitions = {}
-        # self.context = context
 
     @property
     def supports_compile(self):
@@ -146,16 +141,28 @@ class CompilePlatform(Platform):
     def num_threads(self):
         return int(self.config["num_threads"])
 
-    @property
-    def print_output(self):
-        # TODO: get rid of this
-        return (
-            bool(self.config["print_output"])
-            if isinstance(self.config["print_output"], (int, bool))
-            else bool(distutils.util.strtobool(self.config["print_output"]))
+    def get_metrics(self, elf):
+        static_mem = get_static_mem_usage(elf)
+        rom_ro, rom_code, rom_misc, ram_data, ram_zdata = (
+            static_mem["rom_rodata"],
+            static_mem["rom_code"],
+            static_mem["rom_misc"],
+            static_mem["ram_data"],
+            static_mem["ram_zdata"],
         )
+        rom_total = rom_ro + rom_code + rom_misc
+        ram_total = ram_data + ram_zdata
+        metrics = Metrics()
+        metrics.add("Total ROM", rom_total)
+        metrics.add("Total RAM", ram_total)
+        metrics.add("ROM read-only", rom_ro)
+        metrics.add("ROM code", rom_code)
+        metrics.add("ROM misc", rom_misc)
+        metrics.add("RAM data", ram_data)
+        metrics.add("RAM zero-init data", ram_zdata)
+        return metrics
 
-    def generate_elf(self, target, src=None, model=None, num=1, data_file=None):
+    def generate_elf(self, src, target, model=None, num=1, data_file=None):
         raise NotImplementedError
 
     def export_elf(self, path):
@@ -181,6 +188,9 @@ class TargetPlatform(Platform):
 
     REQUIRED = []
 
+    def create_target(self, name):
+        raise NotImplementedError
+
     @property
     def supports_flash(self):
         return True
@@ -189,17 +199,17 @@ class TargetPlatform(Platform):
     def supports_monitor(self):
         return True
 
-    def flash(self, target, timeout=120):
+    def flash(self, elf, target, timeout=120):
         raise NotImplementedError
 
     def monitor(self, target, timeout=60):
         raise NotImplementedError
 
-    def run(self, target, timeout=120):
+    def run(self, elf, target, timeout=120):
         # Only allow one serial communication at a time
         with FileLock(Path(tempfile.gettempdir()) / "mlonmcu_serial.lock"):
 
-            self.flash(target, timeout=timeout)
+            self.flash(elf, target, timeout=timeout)
             output = self.monitor(target, timeout=timeout)
 
         return output
