@@ -22,6 +22,7 @@ from .postprocesses import (
     AverageCyclesPostprocess,
     DetailedCyclesPostprocess,
     FilterColumnsPostprocess,
+    RenameColumnsPostprocess,
     Features2ColumnsPostprocess,
     Config2ColumnsPostprocess,
     VisualizePostprocess,
@@ -32,6 +33,7 @@ SUPPORTED_POSTPROCESSES = {
     "average_cycles": AverageCyclesPostprocess,
     "detailed_cycles": DetailedCyclesPostprocess,
     "filter_cols": FilterColumnsPostprocess,
+    "rename_cols": RenameColumnsPostprocess,
     "features2cols": Features2ColumnsPostprocess,
     "config2cols": Config2ColumnsPostprocess,
     "visualize": VisualizePostprocess,
