@@ -16,26 +16,26 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-import tempfile
-from pathlib import Path
+"""Definition of a MLonMCU Run which represents a single benchmark instance for a given set of options."""
 import os
 import copy
-import numpy as np
+import tempfile
+from pathlib import Path
 from enum import IntEnum
 
 from mlonmcu.logging import get_logger
-from mlonmcu.artifact import ArtifactFormat
+from mlonmcu.artifact import ArtifactFormat, lookup_artifacts
+from mlonmcu.platform.platform import CompilePlatform, TargetPlatform
 from mlonmcu.report import Report  # TODO: move to mlonmcu.session.report
 from mlonmcu.config import resolve_required_config, filter_config
 from mlonmcu.models.lookup import lookup_models
 from mlonmcu.feature.type import FeatureType
-from mlonmcu.feature.features import get_matching_features
+from mlonmcu.feature.features import get_matching_features, get_available_features
 from mlonmcu.target.metrics import Metrics
 from mlonmcu.models import SUPPORTED_FRONTENDS
-
 from mlonmcu.platform import get_platforms
-from mlonmcu.target import SUPPORTED_TARGETS
 from mlonmcu.flow import SUPPORTED_FRAMEWORKS, SUPPORTED_BACKENDS
+
 from .postprocess import SUPPORTED_POSTPROCESSES
 from .postprocess.postprocess import RunPostprocess
 
@@ -58,11 +58,12 @@ class RunStage(IntEnum):
 class Run:
     """A run is single model/backend/framework/target combination with a given set of features and configs."""
 
-    FEATURES = ["autotune"]
+    FEATURES = ["autotune", "target_optimized"]
 
     DEFAULTS = {
         "export_optional": False,
         "tune_enabled": False,
+        "target_to_backend": False,
     }
 
     REQUIRED = []
@@ -77,10 +78,10 @@ class Run:
         idx=None,
         model=None,
         framework=None,
-        frontend=None,
+        frontends=None,
         backend=None,
         target=None,
-        platform=None,  # TODO: rename
+        platforms=None,  # TODO: rename
         features=None,  # TODO: All features combined or explicit run-features -> postprocesses?
         config=None,  # TODO: All config combined or explicit run-config?
         postprocesses=None,
@@ -91,10 +92,10 @@ class Run:
     ):
         self.idx = idx
         self.model = model  # Model name, not object?
-        self.frontend = frontend  # Single one or all enabled ones?
+        self.frontends = frontends if frontends is not None else []
         self.framework = framework  # ???
         self.backend = backend
-        self.platform = platform
+        self.platforms = platforms if platforms is not None else []
         self.artifacts_per_stage = {}
         self.num = num
         self.archived = archived
@@ -102,12 +103,12 @@ class Run:
         self.postprocesses = postprocesses if postprocesses else []
         self.comment = comment
         # self.stage = RunStage.NOP  # max executed stage
-        self.completed = {stage: True if stage == RunStage.NOP else False for stage in RunStage}
+        self.completed = {stage: stage == RunStage.NOP for stage in RunStage}
 
-        self._init_directory()
+        self.init_directory()
         self.target = target
-        self.config = Run.DEFAULTS
-        self.config.update(config if config else {})
+        self.cache_hints = []
+        self.config = config if config else {}
         self.features = features if features else []
         self.run_config = config if config else {}
         self.run_features = self.process_features(features)
@@ -119,6 +120,7 @@ class Run:
         self.report = None
 
     def process_features(self, features):
+        """Utility which handles postprocess_features."""
         if features is None:
             return []
         features = get_matching_features(features, FeatureType.RUN)
@@ -129,34 +131,65 @@ class Run:
 
     @property
     def tune_enabled(self):
+        """Get tune_enabled property."""
         return bool(self.run_config["tune_enabled"])
 
+    @property
+    def target_to_backend(self):
+        """Get target_to_backend property."""
+        return bool(self.run_config["target_to_backend"])
+
+    @property
+    def compile_platform(self):
+        """Get platform for compile stage."""
+        if self.target is not None and isinstance(self.target.platform, CompilePlatform):
+            return self.target.platform
+        for platform in self.platforms:
+            if isinstance(platform, CompilePlatform):
+                return platform
+        return None
+
+    @property
+    def target_platform(self):  # TODO: rename to run_stage?
+        """Get platform for run stage."""
+        if self.target:
+            assert isinstance(self.target.platform, TargetPlatform)
+            return self.target.platform
+        for platform in self.platforms:
+            if isinstance(platform, TargetPlatform):
+                return platform
+        return None
+
     def has_stage(self, stage):
+        """Returns true if the given stage is available for this run."""
         if stage == RunStage.NOP:
             return True
-        elif stage == RunStage.LOAD:
-            return self.model is not None and self.frontend is not None
-        elif stage == RunStage.TUNE:
+        if stage == RunStage.LOAD:
+            return self.model is not None and len(self.frontends) > 0
+        if stage == RunStage.TUNE:
             return self.tune_enabled and self.backend is not None
-        elif stage == RunStage.BUILD:
+        if stage == RunStage.BUILD:
             return self.backend is not None and self.framework is not None
-        elif stage in [RunStage.COMPILE, RunStage.RUN]:
-            return self.target is not None and self.platform is not None
-        elif stage == RunStage.POSTPROCESS:
+        if stage == RunStage.COMPILE:
+            return self.target is not None and len(self.platforms) > 0 and self.compile_platform is not None
+        if stage == RunStage.RUN:
+            return self.target is not None and len(self.platforms) > 0 and self.target_platform is not None
+        if stage == RunStage.POSTPROCESS:
             return len(self.postprocesses) > 0
-        elif stage == RunStage.DONE:
+        if stage == RunStage.DONE:
             return False
-        else:
-            return False  # TODO: Throw error instead?
+        return False  # TODO: Throw error instead?
 
     @property
     def next_stage(self):
+        """Determines the next not yet completed stage. Returns RunStage.DONE if already completed."""
         for stage in RunStage:
             if not self.completed[stage.value] and self.has_stage(stage):
                 return stage
         return RunStage.DONE
 
     def lock(self):
+        """Aquire a mutex to lock the current run."""
         # ret = self.lock.acquire(timeout=0)
         ret = not self.locked
         self.locked = True
@@ -164,10 +197,12 @@ class Run:
             raise RuntimeError("Parallel processing of the same run is not allowed")
 
     def unlock(self):
+        """Release a mutex to unlock the current run."""
         # self.lock.release()
         self.locked = False
 
-    def _init_directory(self):
+    def init_directory(self):
+        """Initialize the temporary directory for this run."""
         if self.session is None:
             assert not self.archived
             self.tempdir = tempfile.TemporaryDirectory()
@@ -179,42 +214,20 @@ class Run:
                 os.mkdir(self.dir)
             # This is not a good idea, but else we would need a mutex/lock on the shared build_dir
             # A solution would be to split up the framework runtime libs from the mlif...
-            if self.platform:
-                self.platform.init_directory(path=Path(self.dir) / self.platform.name)
+            for platform in self.platforms:  # TODO: only do this if needed! (not for every platform)
+                platform.init_directory(path=Path(self.dir) / platform.name)
 
     def copy(self):
+        """Create a new run based on this instance."""
         new = copy.deepcopy(self)
         if self.session:
             new_idx = self.session.request_run_idx()
             new.idx = new_idx
-            self._init_directory()
+            self.init_directory()
         return new
 
-    # TODO: get rid of this
-    # This is currently required as the platforms needs to access the backend, framework and target...
-    def init_platform(self, platform_cls, context=None):
-        required_keys = platform_cls.REQUIRED
-        self.config.update(
-            resolve_required_config(
-                required_keys,
-                features=self.features,
-                config=self.config,
-                cache=context.cache,
-            )
-        )
-        assert self.framework is not None, "Please add a frontend before initializing the Platform"
-        assert self.backend is not None, "Please add a backend before initializing the Platform"
-        platform_config = self.config.copy()
-        platform = platform_cls(
-            features=self.features,
-            config=platform_config,
-        )
-        self.framework.add_platform_defs(platform.name, platform.definitions)
-        self.backend.add_platform_defs(platform.name, platform.definitions)
-        # self.target.add_platform_defs(platform.name, platform.definitions)
-        return platform
-
     def init_component(self, component_cls, context=None):
+        """Helper function to create and configure a MLonMCU component instance for this run."""
         required_keys = component_cls.REQUIRED
         self.config.update(
             resolve_required_config(
@@ -222,47 +235,137 @@ class Run:
                 features=self.features,
                 config=self.config,
                 cache=context.cache if context else None,
+                hints=self.cache_hints,
             )
         )
         component_config = self.config.copy()  # TODOL get rid of this
         return component_cls(features=self.features, config=component_config)
 
     def add_model(self, model):
+        """Setter for the model instance."""
         self.model = model
 
     def add_frontend(self, frontend):
-        self.frontend = frontend
+        """Setter for the frontend instance."""
+        self.frontends = [frontend]
+
+    def add_frontends(self, frontends):
+        """Setter for the list of frontends."""
+        self.frontends = frontends
 
     def add_backend(self, backend):
+        """Setter for the backend instance."""
         self.backend = backend
+        # assert len(self.platforms) > 0, "Add at least a platform before adding a backend."
+        if self.model is not None:
+            assert self.backend.supports_model(self.model), (
+                "The added backend does not support the chosen model."
+                "Add the backend before adding a model to find a suitable frontend."
+            )
+        for platform in self.platforms:
+            self.backend.add_platform_defs(platform.name, platform.definitions)
 
     def add_framework(self, framework):
+        """Setter for the framework instance."""
         self.framework = framework
+        # assert len(self.platforms) > 0, "Add at least a platform before adding a framework."
+        for platform in self.platforms:
+            self.framework.add_platform_defs(platform.name, platform.definitions)
 
     def add_target(self, target):
+        """Setter for the target instance."""
         self.target = target
-        self.target.add_platform_defs(self.platform.name, self.platform.definitions)  # TODO: move to platform?
+        assert self.platforms is not None, "Add at least a platform before adding a target."
+        for platform in self.platforms:
+            self.target.add_platform_defs(platform.name, platform.definitions)
+        self.cache_hints = [self.target.get_arch()]
+        # self.resolve_chache_refs()
 
-    def add_platform(self, platform, context=None):
-        self.platform = platform
+    def add_platform(self, platform):
+        """Setter for the platform instance."""
+        self.platforms = [platform]
+        if self.backend:
+            self.backend.add_platform_defs(platform.name, platform.definitions)
+        if self.framework:
+            self.framework.add_platform_defs(platform.name, platform.definitions)
 
-    def add_postprocesses(self, postprocesses):
-        self.postprocesses = postprocesses
+    def add_platforms(self, platforms):
+        """Setter for the list of platforms."""
+        self.platforms = platforms
+        for platform in platforms:
+            if self.backend:
+                self.backend.add_platform_defs(platform.name, platform.definitions)
+            if self.framework:
+                self.framework.add_platform_defs(platform.name, platform.definitions)
+
+    def add_postprocess(self, postprocess, append=False):
+        """Setter for a postprocess instance."""
+        if append:
+            self.postprocesses.append(postprocess)
+        else:
+            self.postprocesses = [postprocess]
+
+    def add_postprocesses(self, postprocesses, append=False):
+        """Setter for the list of postprocesses."""
+        if append:
+            self.postprocesses.extend(postprocesses)
+        else:
+            self.postprocesses = postprocesses
+
+    def add_feature(self, feature):
+        """Setter for a feature instance."""
+        self.features = [feature]
+        self.run_features = self.process_features(self.features)
+        self.run_config = filter_config(self.run_config, "run", self.DEFAULTS, self.REQUIRED)
+
+    def add_features(self, features, append=False):
+        """Setter for the list of features."""
+        self.features = features if not append else self.features + features
+        self.run_features = self.process_features(self.features)
+        self.run_config = filter_config(self.run_config, "run", self.DEFAULTS, self.REQUIRED)
+
+    def pick_model_frontend(self, model_hints, backend=None):
+        assert len(model_hints) > 0
+        if backend is None:
+            self.frontends = [
+                frontend for frontend in self.frontends if model_hints[0].formats[0] in frontend.input_formats
+            ]
+            assert len(self.frontends) > 0
+            return model_hints[0]
+        for model_hint in model_hints:
+            if backend.supports_model(model_hint):
+                self.frontends = [
+                    frontend for frontend in self.frontends if model_hint.formats[0] in frontend.input_formats
+                ]
+                return model_hint
+        return None
 
     def add_model_by_name(self, model_name, context=None):
+        """Helper function to initialize and configure a model by its name."""
         assert context is not None, "Please supply a context"
-        assert self.frontend is not None, "Add a frontend to the run before adding a model"
-        model_hints = lookup_models([model_name], frontends=[self.frontend], context=context)
+        assert len(self.frontends) > 0, "Add a frontend to the run before adding a model"
+        model_hints = lookup_models([model_name], frontends=self.frontends, context=context)
         assert len(model_hints) > 0, f"Model with name '{model_name}' not found"
-        self.add_model(model_hints[0])
+        model_hint = self.pick_model_frontend(model_hints, backend=self.backend)
+        assert model_hint is not None, "Unable to pick a suitable model"
+        self.add_model(model_hint)
 
     def add_frontend_by_name(self, frontend_name, context=None):
-        assert context is not None and context.environment.has_frontend(
-            frontend_name
-        ), f"The frontend '{frontend_name}' is not enabled for this environment"
-        self.add_frontend(self.init_component(SUPPORTED_FRONTENDS[frontend_name], context=context))
+        """Helper function to initialize and configure a frontend by its name."""
+        self.add_frontends_by_name([frontend_name], context=context)
+
+    def add_frontends_by_name(self, frontend_names, context=None):
+        """Helper function to initialize and configure frontends by their names."""
+        frontends = []
+        for name in frontend_names:
+            assert context is not None and context.environment.has_frontend(
+                name
+            ), f"The frontend '{name}' is not enabled for this environment"
+            frontends.append(self.init_component(SUPPORTED_FRONTENDS[name], context=context))
+        self.add_frontends(frontends)
 
     def add_backend_by_name(self, backend_name, context=None):
+        """Helper function to initialize and configure a backend by its name."""
         assert context is not None and context.environment.has_backend(
             backend_name
         ), f"The backend '{backend_name}' is not enabled for this environment"
@@ -274,37 +377,69 @@ class Run:
         self.add_framework(self.init_component(SUPPORTED_FRAMEWORKS[framework_name], context=context))
 
     def add_target_by_name(self, target_name, context=None):
+        """Helper function to initialize and configure a target by its name."""
+        # We can not use the following code snipped are platform targets may be resolved dynamically
         # assert context is not None and context.environment.has_target(
         #     target_name
         # ), f"The target '{target_name}' is not enabled for this environment"
-        assert self.platform is not None, "Please add a platform to the run before adding the target"
-        self.add_target(self.init_component(self.platform.create_target(target_name), context=context))
+        assert len(self.platforms) > 0, "Please add a platform to the run before adding the target"
+        self.add_target(self.init_component(self.target_platform.create_target(target_name), context=context))
 
     def add_platform_by_name(self, platform_name, context=None):
-        assert context is not None and context.environment.has_platform(
-            platform_name
-        ), f"The platform '{platform_name}' is not enabled for this environment"
-        self.add_platform(self.init_platform(get_platforms()[platform_name], context=context))
+        """Helper function to initialize and configure a platform by its name."""
+        self.add_platforms_by_name([platform_name], context=context)
 
-    def add_postprocesses_by_name(self, postprocess_names, context=None):
-        l = []
-        for postprocess_name in postprocess_names:
+    def add_platforms_by_name(self, platform_names, context=None):
+        """Helper function to initialize and configure platforms by their names."""
+        platforms = []
+        for name in platform_names:
+            assert context is not None and context.environment.has_platform(
+                name
+            ), f"The platform '{name}' is not enabled for this environment"
+            platforms.append(self.init_component(get_platforms()[name], context=context))
+        self.add_platforms(platforms)
+
+    def add_postprocess_by_name(self, postprocess_name, append=True, context=None):
+        """Helper function to initialize and configure a postprocesses by its name."""
+        self.add_postprocesses_by_name([postprocess_name], append=append, context=context)
+
+    def add_postprocesses_by_name(self, postprocess_names, append=False, context=None):
+        """Helper function to initialize and configure postprocesses by their names."""
+        postprocesses = []
+        for name in postprocess_names:
+            # TODO: ?
             # assert context is not None and context.environment.has_postprocess(
             #     postprocess_name
             # ), f"The postprocess '{postprocess_name}' is not enabled for this environment"
-            l.append(self.init_component(SUPPORTED_POSTPROCESSES[postprocess_name], context=context))
-        self.add_postprocesses(l)
+            postprocesses.append(self.init_component(SUPPORTED_POSTPROCESSES[name], context=context))
+        self.add_postprocesses(postprocesses, append=append)
+
+    def add_feature_by_name(self, feature_name, append=True, context=None):
+        """Helper function to initialize and configure a feature by its name."""
+        self.add_features_by_name([feature_name], context=context, append=append)
+
+    def add_features_by_name(self, feature_names, append=False, context=None):
+        """Helper function to initialize and configure features by their names."""
+        features = []
+        for feature_name in feature_names:
+            available_features = get_available_features(feature_name=feature_name)
+            for feature_cls in available_features:
+                features.append(self.init_component(feature_cls, context=context))
+        self.add_features(features, append=append)
 
     @property
     def export_optional(self):
-        return bool(self.config["export_optional"])
+        """Get export_optional property."""
+        return bool(self.run_config["export_optional"])
 
     def __repr__(self):
         probs = []
         if self.model:
             probs.append(str(self.model))
-        if self.frontend:
-            probs.append(str(self.frontend))
+        if len(self.platforms) > 0:
+            probs.append(str(self.platforms[0] if len(self.platforms) == 1 else self.platforms))
+        if len(self.frontends) > 0:
+            probs.append(str(self.frontends[0] if len(self.frontends) == 1 else self.frontends))
         if self.backend:
             probs.append(str(self.backend))
         if self.target:
@@ -317,10 +452,17 @@ class Run:
             probs.append(str(self.config))
         return "Run(" + ",".join(probs) + ")"
 
+    @property
+    def frontend(self):
+        assert len(self.frontends) > 0, "Not frontend is available for this run."
+        return self.frontends[0]
+
     def toDict(self):
-        raise NotImplementedError  # TODO
+        """Utility not implemented yet. (TODO: remove?)"""
+        raise NotImplementedError
 
     def export_stage(self, stage, optional=False, subdir=False):
+        """Export stage artifacts of this run to its directory."""
         # TODO: per stage subdirs?
         if stage in self.artifacts_per_stage:
             artifacts = self.artifacts_per_stage[stage]
@@ -331,11 +473,15 @@ class Run:
                         stage_idx = int(stage)
                         dest = dest / f"stage_{stage_idx}"
                     extract = artifact.fmt == ArtifactFormat.MLF
-                    artifact.export(self.dir, extract=extract)
+                    # extract = artifact.fmt == ArtifactFormat.MLF and not isinstance(self.platform, MicroTvmPlatform)
+                    artifact.export(self.dir)
+                    # Keep the tar as well as the extracted files
+                    if extract:
+                        artifact.export(self.dir, extract=True)
 
-    def postprocess(self, context=None):  # TODO: drop context arguments?
+    def postprocess(self):
         """Postprocess the 'run'."""
-        logger.debug(self.prefix + "Processing stage POSTPROCESS")
+        logger.debug("%s Processing stage POSTPROCESS", self.prefix)
         self.lock()
         # assert self.completed[RunStage.RUN]  # Alternative: allow to trigger previous stages recursively as a fallback
 
@@ -351,24 +497,29 @@ class Run:
         self.completed[RunStage.POSTPROCESS] = True
         self.unlock()
 
-    def run(self, context=None):  # TODO: drop context arguments?
+    def run(self):
         """Run the 'run' using the defined target."""
-        logger.debug(self.prefix + "Processing stage RUN")
+        logger.debug("%s Processing stage RUN", self.prefix)
         self.lock()
         # Alternative: drop artifacts of higher stages when re-triggering a lower one?
-
-        assert self.completed[RunStage.COMPILE]
-        self.export_stage(RunStage.COMPILE, optional=self.export_optional)
-        elf_artifact = self.artifacts_per_stage[RunStage.COMPILE][0]
-        self.target.generate_metrics(elf_artifact.path)
+        if self.has_stage(RunStage.COMPILE):
+            assert self.completed[RunStage.COMPILE]
+            self.export_stage(RunStage.COMPILE, optional=self.export_optional)
+            elf_artifact = self.artifacts_per_stage[RunStage.COMPILE][0]
+            self.target.generate_metrics(elf_artifact.path)
+        else:
+            assert self.completed[RunStage.BUILD]  # Used for tvm platform
+            self.export_stage(RunStage.BUILD, optional=self.export_optional)
+            shared_object_artifact = self.artifacts_per_stage[RunStage.BUILD][0]
+            self.target.generate_metrics(shared_object_artifact.path, num=self.num)
         self.artifacts_per_stage[RunStage.RUN] = self.target.artifacts
 
         self.completed[RunStage.RUN] = True
         self.unlock()
 
-    def compile(self, context=None):
+    def compile(self):
         """Compile the target software for the run."""
-        logger.debug(self.prefix + "Processing stage COMPILE")
+        logger.debug("%s Processing stage COMPILE", self.prefix)
         self.lock()
         assert self.completed[RunStage.BUILD]
 
@@ -379,18 +530,25 @@ class Run:
             if artifact.name == "data.c":
                 artifact.export(self.dir)
                 data_file = Path(self.dir) / "data.c"
-        self.platform.generate_elf(self.target, codegen_dir, num=self.num, data_file=data_file)
-        self.artifacts_per_stage[RunStage.COMPILE] = self.platform.artifacts
+        self.compile_platform.generate_elf(codegen_dir, self.target, num=self.num, data_file=data_file)
+        self.artifacts_per_stage[RunStage.COMPILE] = self.compile_platform.artifacts
 
         self.completed[RunStage.COMPILE] = True
         self.unlock()
 
-    def build(self, context=None):
+    def build(self):
         """Process the run using the choosen backend."""
-        logger.debug(self.prefix + "Processing stage BUILD")
+        logger.debug("%s Processing stage BUILD", self.prefix)
         self.lock()
         assert (not self.has_stage(RunStage.TUNE)) or self.completed[RunStage.TUNE]
 
+        if self.target_to_backend:
+            assert self.target is not None, "Config target_to_backend can only be used if a target was provided"
+            cfg = self.target.get_backend_config(self.backend.name)  # Do not expect a backend prefix here
+            if len(cfg) > 0:
+                logger.debug("Updating backend config based on given target.")
+                self.backend.config.update(cfg)
+
         self.export_stage(RunStage.LOAD, optional=self.export_optional)  # Not required anymore?
         model_artifact = self.artifacts_per_stage[RunStage.LOAD][0]
         if not model_artifact.exported:
@@ -403,7 +561,7 @@ class Run:
                 tuning_artifact = self.artifacts_per_stage[RunStage.TUNE][0]
                 if not tuning_artifact.exported:
                     tuning_artifact.export(self.dir)
-                self.backend.set_tuning_records(tuning_artifact.path)
+                self.backend.tuning_records = tuning_artifact.path
 
         # TODO: allow raw data as well as filepath in backends
         self.backend.generate_code()
@@ -412,9 +570,9 @@ class Run:
         self.completed[RunStage.BUILD] = True
         self.unlock()
 
-    def tune(self, context=None):
+    def tune(self):
         """Tune the run using the choosen backend (if supported)."""
-        logger.debug(self.prefix + "Processing stage TUNE")
+        logger.debug("%s Processing stage TUNE", self.prefix)
         self.lock()
         assert self.completed[RunStage.LOAD]
 
@@ -438,9 +596,9 @@ class Run:
         self.completed[RunStage.TUNE] = True
         self.unlock()
 
-    def load(self, context=None):
+    def load(self):
         """Load the model using the given frontend."""
-        logger.debug(self.prefix + "Processing stage LOAD")
+        logger.debug("%s Processing stage LOAD", self.prefix)
         self.lock()
         # assert self.completed[RunStage.NOP]
 
@@ -453,8 +611,10 @@ class Run:
                 component, name = key.split(".")[:2]
                 if self.backend is not None and component == self.backend.name:
                     self.backend.config[name] = value
-                elif self.platform is not None and component == self.platform.name:
-                    self.platform.config[name] = value
+                else:
+                    for platform in self.platforms:
+                        if platform is not None and component == platform.name:
+                            platform.config[name] = value
                 self.config[key] = value
         self.artifacts_per_stage[RunStage.LOAD] = self.frontend.artifacts
         if data_artifact:
@@ -463,19 +623,21 @@ class Run:
         self.completed[RunStage.LOAD] = True
         self.unlock()
 
-    def process(self, until=RunStage.RUN, skip=[], export=False, context=None):
+    def process(self, until=RunStage.RUN, skip=None, export=False):
         """Process the run until a given stage."""
+        skip = skip if skip is not None else []
         if until == RunStage.DONE:
             until = RunStage.DONE - 1
         start = self.next_stage  # self.stage hold the max finished stage
         if until < start:
-            logger.debug(self.prefix + "Nothing to do")
+            logger.debug("%s Nothing to do", self.prefix)
             return self.get_report()
 
         if start > RunStage.NOP:
             logger.debug(
                 # self.prefix + "Processing run until stage %s: %s",
-                self.prefix + "Continuing run from stage %s until stage %s",
+                "%s Continuing run from stage %s until stage %s",
+                self.prefix,
                 str(RunStage(start).name),
                 str(RunStage(until).name),
                 # str(self),
@@ -483,7 +645,8 @@ class Run:
         else:
             logger.debug(
                 # self.prefix + "Processing run until stage %s: %s",
-                self.prefix + "Processing run until stage %s",
+                "%s Processing run until stage %s",
+                self.prefix,
                 str(RunStage(until).name),
                 # str(self),
             )
@@ -491,7 +654,7 @@ class Run:
             if not self.has_stage(stage):
                 continue
             if stage in skip:
-                logger.debug(self.prefix + "Skipping stage %s", str(RunStage(stage).name))
+                logger.debug("%s Skipping stage %s", self.prefix, str(RunStage(stage).name))
                 continue
             if stage < RunStage.POSTPROCESS:
                 self.report = None  # Regenerate report if earlier stages are executed
@@ -508,14 +671,14 @@ class Run:
             if func:
                 self.failing = False
                 try:
-                    func(context=context)
+                    func()
                 except Exception as e:
                     self.failing = True
                     if self.locked:
                         self.unlock()
                     logger.exception(e)
                     run_stage = RunStage(stage).name
-                    logger.error(self.prefix + f"Run failed at stage '{run_stage}', aborting...")
+                    logger.error("%s Run failed at stage '%s', aborting...", self.prefix, run_stage)
                     break
             # self.stage = stage  # FIXME: The stage_func should update the stage intead?
         report = self.get_report()
@@ -526,30 +689,50 @@ class Run:
         return report
 
     def write_run_file(self):
-        logger.debug(self.prefix + "Writing run file")
+        """Create a run.txt file which contains information used to reconstruct the run based
+        on its properties at a later point in time."""
+        logger.debug("%s Writing run file", self.prefix)
         filename = self.dir / "run.txt"
-        with open(filename, "w") as handle:
+        with open(filename, "w", encoding="utf-8") as handle:
             handle.write(str(self))
 
     @property
     def prefix(self):
+        """Get prefix property."""
         return (
-            (f"[session-{self.session.idx}] [run-{self.idx}] " if self.session else f"[run-{self.idx}]")
+            (f"[session-{self.session.idx}] [run-{self.idx}]" if self.session else f"[run-{self.idx}]")
             if self.idx is not None
             else ""
         )
 
     def get_all_feature_names(self):
+        """Return list of feature names for this run."""
         return [feature.name for feature in self.features]
 
     def get_all_postprocess_names(self):
+        """Return list of postprocess names for this run."""
         return [postprocess.name for postprocess in self.postprocesses]
 
+    def get_frontend_name(self):
+        """Return frontend name(s) for this run."""
+        ret = [frontend.name for frontend in self.frontends]
+        return ret[0] if len(ret) == 1 else ret
+
+    def get_platform_name(self):
+        """Return platform name(s) for this run."""
+        used = list(set([self.compile_platform, self.target_platform]))  # TODO: build_platform, tune_platform
+        ret = [platform.name for platform in used]
+        return ret[0] if len(ret) == 1 else ret
+
     def get_all_configs(self, omit_paths=False, omit_defaults=False, omit_globals=False):
+        """Return dict with component-specific and global configuration for this run."""
+
         def has_prefix(key):
+            """Returns true if the configuration key does not have global scope."""
             return "." in key
 
         def config_helper(obj, prefix=None):
+            """Helper to access the configuration of a given component object."""
             if prefix:
                 name = prefix
             else:
@@ -578,21 +761,22 @@ class Run:
             ret.update(
                 {key: value for key, value in self.config.items() if not has_prefix(key)}
             )  # Only config without a prefix!
-        if self.frontend:
-            ret.update(config_helper(self.frontend))
+        for frontend in self.frontends:
+            ret.update(config_helper(frontend))
         if self.backend:
             ret.update(config_helper(self.backend))
         if self.framework:
             ret.update(config_helper(self.framework))
         if self.target:
             ret.update(config_helper(self.target))
-        if self.platform:
-            ret.update(config_helper(self.platform))
+        for platform in self.platforms:
+            ret.update(config_helper(platform))
         for postprocess in self.postprocesses:
             ret.update(config_helper(postprocess))
         return ret
 
     def get_report(self):
+        """Returns teh complete report of this run."""
         if self.completed[RunStage.POSTPROCESS]:
             if self.report is not None:
                 return (
@@ -607,14 +791,14 @@ class Run:
             pre["Run"] = self.idx
         if self.model:
             pre["Model"] = self.model.name
-        if self.frontend:
-            pre["Frontend"] = self.frontend.name
+        if len(self.frontends) > 0:
+            pre["Frontend"] = self.get_frontend_name()
         if self.framework:
             pre["Framework"] = self.framework.name
         if self.backend:
             pre["Backend"] = self.backend.name
-        if self.platform:
-            pre["Platform"] = self.platform.name
+        if len(self.platforms) > 0:
+            pre["Platform"] = self.get_platform_name()
         if self.target:
             pre["Target"] = self.target.name
         pre["Num"] = self.num
@@ -623,20 +807,40 @@ class Run:
         post["Config"] = self.get_all_configs(omit_paths=True, omit_defaults=True, omit_globals=True)
         post["Postprocesses"] = self.get_all_postprocess_names()
         post["Comment"] = self.comment if len(self.comment) > 0 else "-"
+        if self.failing:
+            post["Failing"] = True
+
         self.export_stage(RunStage.RUN, optional=self.export_optional)
+
+        metrics = Metrics()
+        if RunStage.COMPILE in self.artifacts_per_stage:
+            if len(self.artifacts_per_stage[RunStage.COMPILE]) > 1:  # TODO: look for artifact of type metrics instead
+                compile_metrics_artifact = lookup_artifacts(
+                    self.artifacts_per_stage[RunStage.COMPILE], name="metrics.csv"
+                )[0]
+                compile_metrics = Metrics.from_csv(compile_metrics_artifact.content)
+                metrics = compile_metrics
+
         if RunStage.RUN in self.artifacts_per_stage:
-            metrics_artifact = self.artifacts_per_stage[RunStage.RUN][0]
-            metrics = Metrics.from_csv(metrics_artifact.content)
-        else:
-            metrics = Metrics()
+            run_metrics_artifact = lookup_artifacts(self.artifacts_per_stage[RunStage.RUN], name="metrics.csv")[0]
+            run_metrics = Metrics.from_csv(run_metrics_artifact.content)
+            # Combine with compile metrics
+            metrics_data = metrics.get_data()
+            run_metrics_data = run_metrics.get_data()
+            for key, value in metrics_data.items():
+                if key not in run_metrics_data:
+                    run_metrics.add(key, value)
+            metrics = run_metrics
+
         main = metrics.get_data(include_optional=self.export_optional)
         report.set(pre=[pre], main=[main] if len(main) > 0 else {"Incomplete": [True]}, post=[post])
         return report
 
     def export(self, path=None, optional=False):
         """Write a run configuration to a disk."""
-        logger.debug(self.prefix + "Exporting run to disk")
-        # TODO use proper locks instead of boolean and also lock during export!
+        logger.debug("%s Exporting run to disk", self.prefix)
+        if path is not None:
+            raise NotImplementedError
         assert not self.locked
         for stage in range(self.next_stage):
             if not self.has_stage(stage):
