@@ -16,32 +16,32 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-import multiprocessing
-from time import sleep
-from datetime import datetime
-import pandas as pd
-
-from enum import Enum
-from datetime import datetime
-import tempfile
-from pathlib import Path
+"""Definition of a MLonMCU Run which represents a set of benchmarks in a session."""
 import os
-import logging
+import shutil
+import tempfile
+import multiprocessing
+from datetime import datetime
+from enum import Enum
+from pathlib import Path
 import concurrent.futures
-from tqdm import tqdm
-from .run import RunStage
 
+from tqdm import tqdm
 
 from mlonmcu.session.run import Run
 from mlonmcu.logging import get_logger
 from mlonmcu.report import Report
+from mlonmcu.config import filter_config
 
 from .postprocess.postprocess import SessionPostprocess
+from .run import RunStage
 
 logger = get_logger()  # TODO: rename to get_mlonmcu_logger
 
 
-class SessionStatus(Enum):
+class SessionStatus(Enum):  # TODO: remove?
+    """Status type for a session."""
+
     CREATED = 0
     OPEN = 1
     CLOSED = 2
@@ -49,14 +49,25 @@ class SessionStatus(Enum):
 
 
 class Session:
-    def __init__(self, label="unnamed", idx=None, archived=False, dir=None):
-        timestamp = datetime.now().strftime("%Y%m%dT%H%M%S")
-        self.label = label + "_" + timestamp
+    """A session which wraps around multiple runs in a context."""
+
+    DEFAULTS = {
+        "report_fmt": "csv",
+    }
+
+    def __init__(self, label="", idx=None, archived=False, dir=None, config=None):
+        self.timestamp = datetime.now().strftime("%Y%m%dT%H%M%S")
+        self.label = (
+            label if len(label) > 0 else ("unnamed" + "_" + self.timestamp)
+        )  # TODO: decide if named sessions should also get a timestamp?
         self.idx = idx
+        self.config = config if config else {}
+        self.config = filter_config(self.config, "session", self.DEFAULTS, [])
         self.status = SessionStatus.CREATED
         self.opened_at = None
         self.closed_at = None
         self.runs = []
+        self.report = None
         self.next_run_idx = 0
         self.archived = archived
         if dir is None:
@@ -76,9 +87,16 @@ class Session:
 
     @property
     def prefix(self):
+        """get prefix property."""
         return f"[session-{self.idx}] " if self.idx else ""
 
+    @property
+    def report_fmt(self):
+        """get report_fmt property."""
+        return str(self.config["report_fmt"])
+
     def create_run(self, *args, **kwargs):
+        """Factory method to create a run and add it to this session."""
         idx = len(self.runs)
         logger.debug("Creating a new run with id %s", idx)
         run = Run(*args, idx=idx, session=self, **kwargs)
@@ -94,12 +112,17 @@ class Session:
     #      pass
 
     def get_reports(self):
+        """Returns a full report which includes all runs in this session."""
+        if self.report:
+            return self.report
+
         reports = [run.get_report() for run in self.runs]
         merged = Report()
         merged.add(reports)
         return merged
 
     def enumerate_runs(self):
+        """Update run indices."""
         # Find start index
         max_idx = -1
         for run in self.runs:
@@ -109,11 +132,12 @@ class Session:
         for run in self.runs:
             if not run.archived:
                 run.idx = run_idx
-                run._init_directory()
+                run.init_directory()
                 run_idx += 1
         self.next_run_idx = run_idx
 
     def request_run_idx(self):
+        """Return next free run index."""
         ret = self.next_run_idx
         self.next_run_idx += 1
         # TODO: find a better approach for this
@@ -128,24 +152,26 @@ class Session:
         export=False,
         context=None,
     ):
+        """Process a runs in this session until a given stage."""
 
         # TODO: Add configurable callbacks for stage/run complete
 
         self.enumerate_runs()
+        self.report = None
         assert num_workers > 0, "num_workers can not be < 1"
         workers = []
-        results = []
+        # results = []
         workers = []
-        pbar = None
-        pbar2 = None
+        pbar = None  # Outer progress bar
+        pbar2 = None  # Inner progress bar
         num_runs = len(self.runs)
         num_failures = 0
         stage_failures = {}
         worker_run_idx = []
 
         def _init_progress(total, msg="Processing..."):
-            global pbar
-            pbar = tqdm(
+            """Helper function to initialize a progress bar for the session."""
+            return tqdm(
                 total=total,
                 desc=msg,
                 ncols=100,
@@ -153,39 +179,23 @@ class Session:
                 leave=None,
             )
 
-        def _update_progress(count=1):
-            global pbar
+        def _update_progress(pbar, count=1):
+            """Helper function to update the progress bar for the session."""
             pbar.update(count)
 
-        def _close_progress():
-            global pbar
+        def _close_progress(pbar):
+            """Helper function to close the session progressbar, if available."""
             if pbar:
                 pbar.close()
 
-        def _init_progress2(total, msg="Processing..."):
-            global pbar2
-            pbar2 = tqdm(
-                total=total,
-                desc=msg,
-                ncols=100,
-                bar_format="{l_bar}{bar}| {n_fmt}/{total_fmt} [{elapsed}s]",
-            )
-
-        def _update_progress2(count=1):
-            global pbar2
-            pbar2.update(count)
-
-        def _close_progress2():
-            global pbar2
-            if pbar2:
-                pbar2.close()
-
-        def _process(run, until, skip):
-            run.process(until=until, skip=skip, export=export, context=context)
+        def _process(pbar, run, until, skip):
+            """Helper function to invoke the run."""
+            run.process(until=until, skip=skip, export=export)
             if progress:
-                _update_progress()
+                _update_progress(pbar)
 
         def _join_workers(workers):
+            """Helper function to collect all worker threads."""
             nonlocal num_failures
             results = []
             for i, w in enumerate(workers):
@@ -204,10 +214,11 @@ class Session:
                     else:
                         stage_failures[failed_stage] = [run_index]
             if progress:
-                _close_progress()
+                _close_progress(pbar)
             return results
 
         def _used_stages(runs, until):
+            """Determines the stages which are used by at least one run."""
             used = []
             for stage_index in list(range(RunStage.LOAD, until + 1)) + [RunStage.POSTPROCESS]:
                 stage = RunStage(stage_index)
@@ -221,66 +232,74 @@ class Session:
         with concurrent.futures.ThreadPoolExecutor(num_workers) as executor:
             if per_stage:
                 if progress:
-                    _init_progress2(len(used_stages), msg=f"Processing stages")
+                    pbar2 = _init_progress(len(used_stages), msg="Processing stages")
                 for stage in used_stages:
                     run_stage = RunStage(stage).name
                     if progress:
-                        _init_progress(len(self.runs), msg=f"Processing stage {run_stage}")
+                        pbar = _init_progress(len(self.runs), msg=f"Processing stage {run_stage}")
                     else:
-                        logger.info(self.prefix + f"Processing stage {run_stage}")
+                        logger.info("%s Processing stage %s", self.prefix, run_stage)
                     for i, run in enumerate(self.runs):
                         if i == 0:
                             total_threads = min(len(self.runs), num_workers)
                             cpu_count = multiprocessing.cpu_count()
-                            if (stage >= RunStage.COMPILE) and run.platform:
-                                total_threads *= run.platform.num_threads
+                            if (stage == RunStage.COMPILE) and run.compile_platform:
+                                total_threads *= run.compile_platform.num_threads
                             if total_threads > 2 * cpu_count:
                                 if pbar2:
                                     print()
                                 logger.warning(
-                                    f"The chosen configuration leads to a maximum of {total_threads} threads being processed which heavily exceeds the available CPU resources ({cpu_count}). It is recommended to lower the value of 'mlif.num_threads'!"
+                                    "The chosen configuration leads to a maximum of %d threads being"
+                                    + " processed which heavily exceeds the available CPU resources (%d)."
+                                    + " It is recommended to lower the value of 'mlif.num_threads'!",
+                                    total_threads,
+                                    cpu_count,
                                 )
                         if run.failing:
-                            logger.warning(f"Skiping stage '{run_stage}' for failed run")
+                            logger.warning("Skiping stage '%s' for failed run", run_stage)
                         else:
                             worker_run_idx.append(i)
-                            workers.append(executor.submit(_process, run, until=stage, skip=skipped_stages))
-                    results = _join_workers(workers)
+                            workers.append(executor.submit(_process, pbar, run, until=stage, skip=skipped_stages))
+                    _join_workers(workers)
                     workers = []
                     worker_run_idx = []
                     if progress:
-                        _update_progress2()
+                        _update_progress(pbar2)
                 if progress:
-                    _close_progress2()
+                    _close_progress(pbar2)
             else:
                 if progress:
-                    _init_progress(len(self.runs), msg="Processing all runs")
+                    pbar = _init_progress(len(self.runs), msg="Processing all runs")
                 else:
                     logger.info(self.prefix + "Processing all stages")
                 for i, run in enumerate(self.runs):
                     if i == 0:
                         total_threads = min(len(self.runs), num_workers)
                         cpu_count = multiprocessing.cpu_count()
-                        if (until >= RunStage.COMPILE) and run.platform.name == "mlif":
+                        if (until >= RunStage.COMPILE) and run.compile_platform.name == "mlif":
                             total_threads *= (
-                                run.platform.num_threads
+                                run.compile_platform.num_threads
                             )  # TODO: This should also be used for non-mlif platforms
                         if total_threads > 2 * cpu_count:
                             if pbar2:
                                 print()
                             logger.warning(
-                                f"The chosen configuration leads to a maximum of {total_threads} being processed which heavily exceeds the available CPU resources (cpu_count). It is recommended to lower the value of 'mlif.num_threads'!"
+                                "The chosen configuration leads to a maximum of %d being processed which"
+                                + " heavily exceeds the available CPU resources (%d)."
+                                + " It is recommended to lower the value of 'mlif.num_threads'!",
+                                total_threads,
+                                cpu_count,
                             )
                     worker_run_idx.append(i)
-                    workers.append(executor.submit(_process, run, until=until, skip=skipped_stages))
-                results = _join_workers(workers)
+                    workers.append(executor.submit(_process, pbar, run, until=until, skip=skipped_stages))
+                _join_workers(workers)
         if num_failures == 0:
             logger.info("All runs completed successfuly!")
-        elif num_failures == 0:
+        elif num_failures == num_runs:
             logger.error("All runs have failed to complete!")
         else:
             num_success = num_runs - num_failures
-            logger.warning(f"{num_success} out or {num_runs} runs completed successfully!")
+            logger.warning("%d out or %d runs completed successfully!", num_success, num_runs)
             summary = "\n".join(
                 [
                     f"\t{stage}: \t{len(failed)} failed run(s): " + " ".join([str(idx) for idx in failed])
@@ -288,11 +307,12 @@ class Session:
                     if len(failed) > 0
                 ]
             )
-            logger.info("Summary:\n" + summary)
+            logger.info("Summary:\n%s", summary)
 
         report = self.get_reports()
         logger.info("Postprocessing session report")
-        # Warning: currently we only support one instance of the same type of postprocess, also it will be applied to all rows!
+        # Warning: currently we only support one instance of the same type of postprocess,
+        # also it will be applied to all rows!
         session_postprocesses = []
         for run in self.runs:
             for postprocess in run.postprocesses:
@@ -306,33 +326,48 @@ class Session:
                     # Postprocess has an artifact: write to disk!
                     logger.debug("Writting postprocess artifact to disk: %s", artifact.name)
                     artifact.export(self.dir)
-        report_file = Path(self.dir) / "report.csv"
+        report_file = Path(self.dir) / f"report.{self.report_fmt}"
         report.export(report_file)
         results_dir = context.environment.paths["results"].path
-        results_file = results_dir / f"{self.label}.csv"
+        results_file = results_dir / f"{self.label}.{self.report_fmt}"
         report.export(results_file)
         logger.info(self.prefix + "Done processing runs")
+        self.report = report
         print_report = True
         if print_report:
-            logger.info("Report:\n" + str(report.df))
+            logger.info("Report:\n%s", str(report.df))
+
+        return num_failures == 0
+
+    def discard(self):
+        """Discard a run and remove its directory."""
+        self.close()
+        if self.dir.is_dir():
+            logger.debug("Cleaning up discarded session")
+            shutil.rmtree(self.dir)
 
     def __repr__(self):
         return f"Session(idx={self.idx},status={self.status},runs={self.runs})"
 
     @property
     def active(self):
+        """Get active property."""
         return self.status == SessionStatus.OPEN
 
     def open(self):
+        """Open this run."""
         self.status = SessionStatus.OPEN
         self.opened_at = datetime.now()
 
     def close(self, err=None):
+        """Close this run."""
         if err:
             self.status = SessionStatus.ERROR
         else:
             self.status = SessionStatus.CLOSED
         self.closed_at = datetime.now()
+        if self.tempdir:
+            self.tempdir.cleanup()
 
 
 # TODO: implement close()? and use closing contextlib? for tempdir
