@@ -134,7 +134,7 @@ def exec_getout(*args, live: bool = False, print_output: bool = True, handle_exi
             assert exit_code == 0, "The process returned an non-zero exit code {}! (CMD: `{}`)".format(
                 exit_code, " ".join(list(map(str, args)))
             )
-        except KeyboardInterrupt as e:
+        except KeyboardInterrupt:
             logger.debug("Interrupted subprocess. Sending SIGINT signal...")
             pid = process.pid
             os.kill(pid, signal.SIGINT)
@@ -154,14 +154,14 @@ def exec_getout(*args, live: bool = False, print_output: bool = True, handle_exi
             assert exit_code == 0, "The process returned an non-zero exit code {}! (CMD: `{}`)".format(
                 exit_code, " ".join(list(map(str, args)))
             )
-        except KeyboardInterrupt as e:
+        except KeyboardInterrupt:
             logger.debug("Interrupted subprocess. Sending SIGINT signal...")
             pid = p.pid
             os.kill(pid, signal.SIGINT)
         except subprocess.CalledProcessError as e:
             outStr = e.output.decode(errors="replace")
             logger.error(outStr)
-            raise
+            raise e
 
     return outStr
 
@@ -216,7 +216,7 @@ def clone(
             repo = Repo.clone_from(url, dest, recursive=recursive, no_checkout=True)
             repo.git.checkout(branch)
             if recursive:
-                output = repo.git.submodule("update", "--init", "--recursive")
+                repo.git.submodule("update", "--init", "--recursive")
         else:
             Repo.clone_from(url, dest, recursive=recursive)
 
@@ -250,7 +250,7 @@ def make(*args, threads=multiprocessing.cpu_count(), use_ninja=False, cwd=None,
     tool = "ninja" if use_ninja else "make"
     extraArgs.append("-j" + str(threads))
     cmd = [tool] + extraArgs + list(args)
-    exec_getout(*cmd, cwd=cwd, print_output=False, **kwargs)
+    return exec_getout(*cmd, cwd=cwd, print_output=False, **kwargs)
 
 
 def cmake(src, *args, debug=False, use_ninja=False, cwd=None, **kwargs):
@@ -264,7 +264,7 @@ def cmake(src, *args, debug=False, use_ninja=False, cwd=None, **kwargs):
     if use_ninja:
         extraArgs.append("-GNinja")
     cmd = ["cmake", str(src)] + extraArgs + list(args)
-    exec_getout(*cmd, cwd=cwd, print_output=False, **kwargs)
+    return exec_getout(*cmd, cwd=cwd, print_output=False, **kwargs)
 
 
 # def move(a, b):  # TODO: make every utility compatible with Paths!
