@@ -18,14 +18,13 @@
 #
 """Helper functions used by MLonMCU targets"""
 
-import logging
 import subprocess
 import argparse
 from typing import List, Callable
 
 from mlonmcu.cli.helper.parse import extract_feature_names, extract_config
 from mlonmcu.feature.type import FeatureType
-from mlonmcu.feature.features import get_available_feature_names, get_available_features
+from mlonmcu.feature.features import get_available_features
 from mlonmcu.logging import get_logger
 
 logger = get_logger()
@@ -37,7 +36,7 @@ def execute(
     ignore_output: bool = False,
     live: bool = False,
     print_func: Callable = print,
-    handle_exit = None,
+    handle_exit=None,
     err_func: Callable = logger.error,
     **kwargs,
 ) -> str:
@@ -97,7 +96,7 @@ def execute(
         if handle_exit is not None:
             exit_code = handle_exit(exit_code)
         if exit_code != 0:
-            err_func(out_Str)
+            err_func(out_str)
         assert exit_code == 0, "The process returned an non-zero exit code {}! (CMD: `{}`)".format(
             exit_code, " ".join(list(map(str, args)))
         )
