@@ -55,10 +55,12 @@ def parseElf(inFile):
         ".comment",
         ".riscv.attributes",
         ".strtab",
+        ".stabstr",
         ".shstrtab",
         ".symtab",
         ".eh_frame",
         ".stab",
+        ".heap",  # ?
         # The following are x86 only:
         ".interp",
         ".dynsym",
@@ -86,6 +88,7 @@ def parseElf(inFile):
         ".table",
         "dummy",
         "heap_start",
+        ".info",
     ]
 
     with open(inFile, "rb") as f:
@@ -102,6 +105,7 @@ def parseElf(inFile):
                 m["rom_rodata"] += s.data_size
             elif s.name in [
                 ".vectors",
+                "iram0.vectors",
                 ".init_array",
                 ".fini_array",
                 ".fini",
