@@ -24,18 +24,19 @@ import csv
 from pathlib import Path
 
 from mlonmcu.logging import get_logger
-
-logger = get_logger()
-
+from mlonmcu.artifact import Artifact, ArtifactFormat
+from mlonmcu.feature.features import SUPPORTED_TVM_BACKENDS
 from .common import cli, execute
 from .riscv import RISCVTarget
 from .metrics import Metrics
 
+logger = get_logger()
+
 
 class EtissPulpinoTarget(RISCVTarget):
     """Target using a Pulpino-like VP running in the ETISS simulator"""
 
-    FEATURES = ["gdbserver", "etissdbg", "trace"]
+    FEATURES = ["gdbserver", "etissdbg", "trace", "log_instrs"]
 
     DEFAULTS = {
         **RISCVTarget.DEFAULTS,
@@ -44,25 +45,33 @@ class EtissPulpinoTarget(RISCVTarget):
         "gdbserver_port": 2222,
         "debug_etiss": False,
         "trace_memory": False,
+        "plugins": [],
         "verbose": False,
-        # TODO: how to keep this in sync with setup/tasks.py? (point to ETISSPulpinoTarget.DEFAULTS?)
+        "cpu_arch": "RISCV",  # For V/P-Extension support, use RV32IMAFDCPV
         "etissvp.rom_start": 0x0,
         "etissvp.rom_size": 0x800000,  # 8 MB
         "etissvp.ram_start": 0x800000,
         "etissvp.ram_size": 0x4000000,  # 64 MB
         "etissvp.cycle_time_ps": 31250,  # 32 MHz
     }
-    REQUIRED = RISCVTarget.REQUIRED + ["etiss.install_dir"]
+    REQUIRED = RISCVTarget.REQUIRED + ["etiss.src_dir", "etiss.install_dir", "etissvp.script"]
 
     def __init__(self, name="etiss_pulpino", features=None, config=None):
         super().__init__(name, features=features, config=config)
-        self.etiss_script = Path(self.etiss_dir) / "examples" / "bare_etiss_processor" / "run_helper.sh"
-        self.metrics_script = Path(self.etiss_dir) / "examples" / "bare_etiss_processor" / "get_metrics.py"
+        self.metrics_script = Path(self.etiss_src_dir) / "src" / "bare_etiss_processor" / "get_metrics.py"
+
+    @property
+    def etiss_src_dir(self):
+        return self.config["etiss.src_dir"]
 
     @property
     def etiss_dir(self):
         return self.config["etiss.install_dir"]
 
+    @property
+    def etiss_script(self):
+        return self.config["etissvp.script"]
+
     @property
     def gdbserver_enable(self):
         return bool(self.config["gdbserver_enable"])
@@ -83,6 +92,10 @@ class EtissPulpinoTarget(RISCVTarget):
     def trace_memory(self):
         return bool(self.config["trace_memory"])
 
+    @property
+    def plugins(self):
+        return self.config["plugins"]
+
     @property
     def verbose(self):
         return bool(self.config["verbose"])
@@ -107,10 +120,18 @@ class EtissPulpinoTarget(RISCVTarget):
     def cycle_time_ps(self):
         return int(self.config["etissvp.cycle_time_ps"])
 
+    @property
+    def cpu_arch(self):
+        return self.config["cpu_arch"]
+
     # TODO: other properties
 
     def write_ini(self, path):
+        # TODO: Either create artifact for ini or prefer to use cmdline args.
         with open(path, "w") as f:
+            if self.cpu_arch:
+                f.write("[StringConfigurations]\n")
+                f.write(f"arch.cpu={self.cpu_arch}\n")
             f.write("[IntConfigurations]\n")
             f.write(f"simple_mem_system.memseg_origin_00={hex(self.rom_start)}\n")
             f.write(f"simple_mem_system.memseg_length_00={hex(self.rom_size)}\n")
@@ -121,6 +142,7 @@ class EtissPulpinoTarget(RISCVTarget):
 
             if self.gdbserver_enable:
                 f.write("[Plugin gdbserver]\n")
+                # This could also be accomplished using `--plugin.gdbserver.port` on the cmdline
                 f.write(f"plugin.gdbserver.port={self.gdbserver_port}")
 
     def exec(self, program, *args, cwd=os.getcwd(), **kwargs):
@@ -148,24 +170,30 @@ class EtissPulpinoTarget(RISCVTarget):
         etiss_ini = os.path.join(cwd, "custom.ini")
         self.write_ini(etiss_ini)
         etiss_script_args.append("-i" + etiss_ini)
+        if len(self.plugins):
+            plugins_str = " ".join(self.plugins)  # TODO: find out separator
+            etiss_script_args.extend(["-p", plugins_str])
 
         if self.timeout_sec > 0:
             raise NotImplementedError
         else:
             ret = execute(
-                self.etiss_script.resolve(),
+                Path(self.etiss_script).resolve(),
                 program,
                 *etiss_script_args,
                 *args,
+                cwd=cwd,
                 **kwargs,
             )
         return ret
 
-    def parse_stdout(self, out):
+    def parse_stdout(self, out, handle_exit=None):
         exit_match = re.search(r"exit called with code: (.*)", out)
         if exit_match:
-            exit_code = exit_match.group(1)
-            if int(exit_code) != 0:
+            exit_code = int(exit_match.group(1))
+            if handle_exit is not None:
+                exit_code = handle_exit(exit_code)
+            if exit_code != 0:
                 logger.error("Execution failed - " + out)
                 raise RuntimeError(f"unexpected exit code: {exit_code}")
         error_match = re.search(r"ETISS: Error: (.*)", out)
@@ -175,20 +203,24 @@ class EtissPulpinoTarget(RISCVTarget):
 
         cpu_cycles = re.search(r"CPU Cycles \(estimated\): (.*)", out)
         if not cpu_cycles:
-            logger.warning("unexpected script output (cycles)")
+            if exit_code == 0:
+                logger.warning("unexpected script output (cycles)")
             cycles = None
         else:
             cycles = int(float(cpu_cycles.group(1)))
         mips_match = re.search(r"MIPS \(estimated\): (.*)", out)
         if not mips_match:
-            raise logger.warning("unexpected script output (mips)")
+            if exit_code == 0:
+                raise logger.warning("unexpected script output (mips)")
             mips = None
         else:
             mips = int(float(mips_match.group(1)))
 
         return cycles, mips
 
-    def get_metrics(self, elf, directory, verbose=False):
+    def get_metrics(self, elf, directory, handle_exit=None, num=None):
+        assert num is None
+        out = ""
         if self.trace_memory:
             trace_file = os.path.join(directory, "dBusAccess.csv")
             if os.path.exists(trace_file):
@@ -200,11 +232,13 @@ class EtissPulpinoTarget(RISCVTarget):
         if os.path.exists(metrics_file):
             os.remove(metrics_file)
 
-        if verbose:
-            out = self.exec(elf, cwd=directory, live=True)
+        if self.print_outputs:
+            out += self.exec(elf, cwd=directory, live=True, handle_exit=handle_exit)
         else:
-            out = self.exec(elf, cwd=directory, live=False, print_func=lambda *args, **kwargs: None)
-        total_cycles, mips = self.parse_stdout(out)
+            out += self.exec(
+                elf, cwd=directory, live=False, print_func=lambda *args, **kwargs: None, handle_exit=handle_exit
+            )
+        total_cycles, mips = self.parse_stdout(out, handle_exit=handle_exit)
 
         get_metrics_args = [elf]
         etiss_ini = os.path.join(directory, "custom.ini")
@@ -213,13 +247,14 @@ class EtissPulpinoTarget(RISCVTarget):
         if trace_file:
             get_metrics_args.extend(["--trace", trace_file])
         get_metrics_args.extend(["--out", metrics_file])
-        if verbose:
-            out2 = execute(self.metrics_script.resolve(), *get_metrics_args, live=True)
+        if self.print_outputs:
+            out += execute(self.metrics_script.resolve(), *get_metrics_args, live=True)
         else:
-            out2 = execute(
+            out += execute(
                 self.metrics_script.resolve(),
                 *get_metrics_args,
                 live=False,
+                cwd=directory,
                 print_func=lambda *args, **kwargs: None,
             )
 
@@ -275,14 +310,30 @@ class EtissPulpinoTarget(RISCVTarget):
                 metrics.add("RAM stack", ram_stack)
                 metrics.add("RAM heap", ram_heap)
 
-        return metrics
+        artifacts = []
+        ini_content = open(etiss_ini, "r").read()
+        ini_artifact = Artifact("custom.ini", content=ini_content, fmt=ArtifactFormat.TEXT)
+        artifacts.append(ini_artifact)
+
+        return metrics, out, artifacts
 
     def get_target_system(self):
         return self.name
 
     def get_platform_defs(self, platform):
+        assert platform == "mlif"
         ret = super().get_platform_defs(platform)
         ret["ETISS_DIR"] = self.etiss_dir
+        ret["PULPINO_ROM_START"] = self.rom_start
+        ret["PULPINO_ROM_SIZE"] = self.rom_size
+        ret["PULPINO_RAM_START"] = self.ram_start
+        ret["PULPINO_RAM_SIZE"] = self.ram_size
+        return ret
+
+    def get_backend_config(self, backend):
+        ret = super().get_backend_config(backend)
+        if backend in SUPPORTED_TVM_BACKENDS:
+            ret.update({"target_model": "etissvp"})
         return ret
 
 
