@@ -20,29 +20,26 @@
 
 import os
 import re
-import csv
 from pathlib import Path
 
-# from mlonmcu.context import MlonMcuContext
 from mlonmcu.logging import get_logger
-
-logger = get_logger()
-
 from .common import cli, execute
 from .riscv import RISCVTarget
 from .metrics import Metrics
-from .elf import get_results
+
+logger = get_logger()
 
 
 class OVPSimTarget(RISCVTarget):
     """Target using an ARM FVP (fixed virtual platform) based on a Cortex M55 with EthosU support"""
 
-    FEATURES = ["vext", "gdbserver"]
+    FEATURES = ["vext", "pext", "gdbserver"]
 
     DEFAULTS = {
         **RISCVTarget.DEFAULTS,
         "vlen": 32,  # vectorization=off
         "enable_vext": False,
+        "enable_pext": False,
         "enable_fpu": True,
         "variant": "RVB32I",
         # "extensions": "MAFDCV",
@@ -63,7 +60,15 @@ class OVPSimTarget(RISCVTarget):
 
     @property
     def extensions(self):
-        return str(self.config["extensions"])
+        ret = str(self.config["extensions"])
+
+        if self.enable_pext:
+            if "P" not in ret:
+                ret += "BP"
+        if self.enable_vext:
+            if "V" not in ret:
+                ret += "V"
+        return ret
 
     @property
     def vlen(self):
@@ -77,12 +82,22 @@ class OVPSimTarget(RISCVTarget):
     def enable_vext(self):
         return bool(self.config["enable_vext"])
 
-    def get_default_ovpsim_args(self):
+    @property
+    def enable_pext(self):
+        return bool(self.config["enable_pext"])
+
+    @property
+    def arch(self):
+        ret = str(self.config["arch"])
+        if self.enable_pext:
+            if "p" not in ret[2:]:
+                ret += "p"
         if self.enable_vext:
-            if "V" not in self.extensions:
-                self.config["extensions"] += "V"
-            if "V" not in self.variant:
-                self.config["variant"] += "V"
+            if "v" not in ret[2:]:
+                ret += "v"
+        return ret
+
+    def get_default_ovpsim_args(self):
         args = [
             "--variant",
             self.variant,
@@ -91,7 +106,17 @@ class OVPSimTarget(RISCVTarget):
             "--override",
             "riscvOVPsim/cpu/unaligned=T",
         ]
+        if self.enable_pext:
+            args.extend(
+                [
+                    "--override",
+                    "riscvOVPsim/cpu/dsp_version=0.9.6",
+                    "--override",
+                    "riscvOVPsim/cpu/bitmanip_version=1.0.0",
+                ]
+            )
         if self.enable_vext:
+            assert self.enable_fpu, "Spike V-Extension requires enabled FPU"
             args.extend(
                 [
                     "--override",
@@ -122,7 +147,7 @@ class OVPSimTarget(RISCVTarget):
         ovpsim_args.extend(self.get_default_ovpsim_args())
 
         if len(self.extra_args) > 0:
-            spike_args.extend(self.extra_args.split(" "))
+            ovpsim_args.extend(self.extra_args.split(" "))
 
         if self.timeout_sec > 0:
             raise NotImplementedError
@@ -136,7 +161,7 @@ class OVPSimTarget(RISCVTarget):
         return ret
 
     def parse_stdout(self, out):
-        cpi = 1
+        # cpi = 1
         cpu_cycles = re.search(r"  Simulated instructions:(.*)", out)
         if not cpu_cycles:
             raise RuntimeError("unexpected script output (cycles)")
@@ -151,35 +176,22 @@ class OVPSimTarget(RISCVTarget):
                 mips = float(mips_str)
         return cycles, mips
 
-    def get_metrics(self, elf, directory, verbose=False):
-        if verbose:
-            out = self.exec(elf, cwd=directory, live=True)
+    def get_metrics(self, elf, directory, handle_exit=None, num=None):
+        assert num is None
+        out = ""
+        if self.print_outputs:
+            out += self.exec(elf, cwd=directory, live=True, handle_exit=handle_exit)
         else:
-            out = self.exec(elf, cwd=directory, live=False, print_func=lambda *args, **kwargs: None)
+            out += self.exec(
+                elf, cwd=directory, live=False, print_func=lambda *args, **kwargs: None, handle_exit=handle_exit
+            )
         cycles, mips = self.parse_stdout(out)
 
         metrics = Metrics()
         metrics.add("Total Cycles", cycles)
         metrics.add("MIPS", cycles, optional=True)
-        static_mem = get_results(elf)
-        rom_ro, rom_code, rom_misc, ram_data, ram_zdata = (
-            static_mem["rom_rodata"],
-            static_mem["rom_code"],
-            static_mem["rom_misc"],
-            static_mem["ram_data"],
-            static_mem["ram_zdata"],
-        )
-        rom_total = rom_ro + rom_code + rom_misc
-        ram_total = ram_data + ram_zdata
-        metrics.add("Total ROM", rom_total)
-        metrics.add("Total RAM", ram_total)
-        metrics.add("ROM read-only", rom_ro)
-        metrics.add("ROM code", rom_code)
-        metrics.add("ROM misc", rom_misc)
-        metrics.add("RAM data", ram_data)
-        metrics.add("RAM zero-init data", ram_zdata)
 
-        return metrics
+        return metrics, out, []
 
 
 if __name__ == "__main__":
