@@ -20,29 +20,30 @@
 
 import os
 import re
-import csv
 from pathlib import Path
 
-# from mlonmcu.context import MlonMcuContext
 from mlonmcu.logging import get_logger
-
-logger = get_logger()
-
+from mlonmcu.config import str2bool
+from mlonmcu.feature.features import SUPPORTED_TVM_BACKENDS
 from .common import cli, execute
 from .riscv import RISCVTarget
 from .metrics import Metrics
-from .elf import get_results
+
+logger = get_logger()
 
 
 class SpikeTarget(RISCVTarget):
-    """Target using an ARM FVP (fixed virtual platform) based on a Cortex M55 with EthosU support"""
+    """Target using the riscv-isa-sim (Spike) RISC-V simulator."""
 
-    FEATURES = ["vext"]
+    FEATURES = ["vext", "pext", "cachesim", "log_instrs"]
 
     DEFAULTS = {
         **RISCVTarget.DEFAULTS,
         "enable_vext": False,
+        "enable_pext": False,
         "vlen": 0,  # vectorization=off
+        "spikepk_extra_args": [],
+        "end_to_end_cycles": False,
     }
     REQUIRED = RISCVTarget.REQUIRED + ["spike.exe", "spike.pk"]
 
@@ -57,34 +58,53 @@ class SpikeTarget(RISCVTarget):
     def spike_pk(self):
         return Path(self.config["spike.pk"])
 
-    @property
-    def extra_args(self):
-        return str(self.config["extra_args"])
-
     @property
     def enable_vext(self):
         return bool(self.config["enable_vext"])
 
+    @property
+    def enable_pext(self):
+        return bool(self.config["enable_pext"])
+
+    @property
+    def arch(self):
+        ret = str(self.config["arch"])
+        if self.enable_pext:
+            if "p" not in ret[2:]:
+                ret += "p"
+        if self.enable_vext:
+            if "v" not in ret[2:]:
+                ret += "v"
+
+        return ret
+
     @property
     def vlen(self):
         return int(self.config["vlen"])
 
     @property
-    def enable_vext(self):
-        return bool(self.config["enable_vext"])
+    def spikepk_extra_args(self):
+        return self.config["spikepk_extra_args"]
+
+    @property
+    def end_to_end_cycles(self):
+        return str2bool(self.config["end_to_end_cycles"])
 
     def exec(self, program, *args, cwd=os.getcwd(), **kwargs):
         """Use target to execute a executable with given arguments"""
         spike_args = []
-
-        if self.enable_vext:
-            if "c" not in self.arch:
-                self.config["arch"] += "v"
+        spikepk_args = []
 
         spike_args.append(f"--isa={self.arch}")
 
         if len(self.extra_args) > 0:
-            spike_args.extend(self.extra_args.split(" "))
+            spike_args.extend(self.extra_args)
+
+        if self.end_to_end_cycles:
+            spikepk_args.append("-s")
+
+        if len(self.spikepk_extra_args) > 0:
+            spikepk_args.extend(self.spikepk_extra_args.split(" "))
 
         if self.enable_vext:
             assert self.vlen > 0
@@ -99,6 +119,7 @@ class SpikeTarget(RISCVTarget):
             self.spike_exe.resolve(),
             *spike_args,
             self.spike_pk.resolve(),
+            *spikepk_args,
             program,
             *args,
             **kwargs,
@@ -106,43 +127,41 @@ class SpikeTarget(RISCVTarget):
         return ret
 
     def parse_stdout(self, out):
-        cpu_cycles = re.search(r"Total Cycles: (.*)", out)
+        if self.end_to_end_cycles:
+            cpu_cycles = re.search(r"(\d*) cycles", out)
+        else:
+            cpu_cycles = re.search(r"Total Cycles: (.*)", out)
         if not cpu_cycles:
             logger.warning("unexpected script output (cycles)")
             cycles = None
         else:
             cycles = int(float(cpu_cycles.group(1)))
-        mips = None  # TODO: parse mips?
+        # mips = None  # TODO: parse mips?
         return cycles
 
-    def get_metrics(self, elf, directory, verbose=False):
-        if verbose:
-            out = self.exec(elf, cwd=directory, live=True)
+    def get_metrics(self, elf, directory, handle_exit=None, num=None):
+        assert num is None
+        out = ""
+        if self.print_outputs:
+            out += self.exec(elf, cwd=directory, live=True, handle_exit=handle_exit)
         else:
-            out = self.exec(elf, cwd=directory, live=False, print_func=lambda *args, **kwargs: None)
+            out += self.exec(
+                elf, cwd=directory, live=False, print_func=lambda *args, **kwargs: None, handle_exit=handle_exit
+            )
         cycles = self.parse_stdout(out)
 
         metrics = Metrics()
         metrics.add("Total Cycles", cycles)
-        static_mem = get_results(elf)
-        rom_ro, rom_code, rom_misc, ram_data, ram_zdata = (
-            static_mem["rom_rodata"],
-            static_mem["rom_code"],
-            static_mem["rom_misc"],
-            static_mem["ram_data"],
-            static_mem["ram_zdata"],
-        )
-        rom_total = rom_ro + rom_code + rom_misc
-        ram_total = ram_data + ram_zdata
-        metrics.add("Total ROM", rom_total)
-        metrics.add("Total RAM", ram_total)
-        metrics.add("ROM read-only", rom_ro)
-        metrics.add("ROM code", rom_code)
-        metrics.add("ROM misc", rom_misc)
-        metrics.add("RAM data", ram_data)
-        metrics.add("RAM zero-init data", ram_zdata)
 
-        return metrics
+        return metrics, out, []
+
+    def get_backend_config(self, backend):
+        ret = super().get_backend_config(backend)
+        if backend in SUPPORTED_TVM_BACKENDS:
+            ret.update({"target_model": "spike-rv32"})
+            if self.enable_pext:
+                pass  # TODO: change graph layout to use SIMD kernels
+        return ret
 
 
 if __name__ == "__main__":
