@@ -30,12 +30,12 @@ from mlonmcu.feature.feature import Feature
 from mlonmcu.feature.type import FeatureType
 from mlonmcu.feature.features import get_matching_features
 from mlonmcu.artifact import Artifact, ArtifactFormat
+from mlonmcu.config import str2bool
 
 
 # TODO: class TargetFactory:
 from .common import execute
 from .metrics import Metrics
-from .elf import get_results
 
 
 class Target:
@@ -58,7 +58,10 @@ class Target:
     """
 
     FEATURES = []
-    DEFAULTS = {}
+    DEFAULTS = {
+        "print_outputs": False,
+    }
+
     REQUIRED = []
 
     def __init__(
@@ -69,6 +72,7 @@ class Target:
     ):
         self.name = name
         self.config = config if config else {}
+        self.callbacks = []
         self.features = self.process_features(features)
         self.config = filter_config(self.config, self.name, self.DEFAULTS, self.REQUIRED)
         self.inspect_program = "readelf"
@@ -76,6 +80,10 @@ class Target:
         self.env = os.environ
         self.artifacts = []
 
+    @property
+    def print_outputs(self):
+        return str2bool(self.config["print_outputs"])
+
     def __repr__(self):
         return f"Target({self.name})"
 
@@ -86,9 +94,10 @@ class Target:
         for feature in features:
             assert feature.name in self.FEATURES, f"Incompatible feature: {feature.name}"
             feature.add_target_config(self.name, self.config)
+            feature.add_target_callback(self.name, self.callbacks)
         return features
 
-    def exec(self, program: Path, *args, **kwargs):
+    def exec(self, program: Path, *args, cwd=os.getcwd(), **kwargs):
         """Use target to execute a executable with given arguments"""
         raise NotImplementedError
 
@@ -96,13 +105,16 @@ class Target:
         """Use target to inspect a executable"""
         return execute(self.inspect_program, program, *self.inspect_program_args, *args, **kwargs)
 
-    def get_metrics(self, elf, directory, verbose=True):
+    def get_metrics(self, elf, directory, handle_exit=None, num=None):
+        assert num is None
         # This should not be accurate, just a fallback which should be overwritten
         start_time = time.time()
-        if verbose:
-            out = self.exec(elf, cwd=directory, live=True)
+        if self.print_outputs:
+            out = self.exec(elf, cwd=directory, live=True, handle_exit=handle_exit)
         else:
-            out = self.exec(elf, cwd=directory, live=False, print_func=lambda *args, **kwargs: None)
+            out = self.exec(
+                elf, cwd=directory, live=False, print_func=lambda *args, **kwargs: None, handle_exit=handle_exit
+            )
         # TODO: do something with out?
         end_time = time.time()
         diff = end_time - start_time
@@ -110,33 +122,23 @@ class Target:
         metrics = Metrics()
         metrics.add("Runtime [s]", diff)
 
-        static_mem = get_results(elf)
-        rom_ro, rom_code, rom_misc, ram_data, ram_zdata = (
-            static_mem["rom_rodata"],
-            static_mem["rom_code"],
-            static_mem["rom_misc"],
-            static_mem["ram_data"],
-            static_mem["ram_zdata"],
-        )
-        rom_total = rom_ro + rom_code + rom_misc
-        ram_total = ram_data + ram_zdata
-        metrics.add("Total ROM", rom_total)
-        metrics.add("Total RAM", ram_total)
-        metrics.add("ROM read-only", rom_ro)
-        metrics.add("ROM code", rom_code)
-        metrics.add("ROM misc", rom_misc)
-        metrics.add("RAM data", ram_data)
-        metrics.add("RAM zero-init data", ram_zdata)
-        return metrics
+        return metrics, out, []
 
-    def generate_metrics(self, elf, verbose=False):
+    def generate_metrics(self, elf, num=None):
         artifacts = []
         with tempfile.TemporaryDirectory() as temp_dir:
-            metrics = self.get_metrics(elf, temp_dir, verbose=verbose)
+            metrics, out, artifacts_ = self.get_metrics(elf, temp_dir, num=num)
+            artifacts.extend(artifacts_)
+            for callback in self.callbacks:  # TODO: give priorities to determine order?
+                callback(out, metrics, artifacts)
             content = metrics.to_csv(include_optional=True)  # TODO: store df instead?
             artifact = Artifact("metrics.csv", content=content, fmt=ArtifactFormat.TEXT)
             # Alternative: artifact = Artifact("metrics.csv", data=df/dict, fmt=ArtifactFormat.DATA)
             artifacts.append(artifact)
+        stdout_artifact = Artifact(
+            f"{self.name}_out.log", content=out, fmt=ArtifactFormat.TEXT
+        )  # TODO: rename to tvmaot_out.log?
+        artifacts.append(stdout_artifact)
         self.artifacts = artifacts
 
     def export_metrics(self, path):
@@ -166,3 +168,6 @@ class Target:
 
     def add_platform_defs(self, platform, defs):
         defs.update(self.get_platform_defs(platform))
+
+    def get_backend_config(self, backend):
+        return {}
