@@ -50,10 +50,11 @@ from mlonmcu.models.lookup import print_summary
 # def test_models_print_groups():
 #     pass
 
+
 def _create_fake_metadata(path, base="metadata", ext="yaml"):
     metadata_file = path / f"{base}.{ext}"
     dummy_data = {}
-    with open(metadata_file.resolve(), 'w') as handle:
+    with open(metadata_file.resolve(), "w") as handle:
         yaml.dump(dummy_data, handle)
 
 
@@ -73,19 +74,31 @@ def _create_fake_models(path, names, ext="tflite", with_subdir=True, with_metada
             else:
                 _create_fake_metadata(model_dir, base=name)
 
+
 def _create_fake_modelgroups(path, data={}, base="groups", ext="yaml"):
     path.mkdir(exist_ok=True)
     groups_file = path / f"{base}.{ext}"
 
-    with open(groups_file.resolve(), 'w') as handle:
+    with open(groups_file.resolve(), "w") as handle:
         yaml.dump(data, handle)
 
 
-
-def _check_summary_output(out, expected_paths=[], expected_models=[], expected_groups={}, expected_skipped=0, expected_metadata_count=0, expected_duplicates=0, expected_group_duplicates=0, detailed=False):
+def _check_summary_output(
+    out,
+    expected_paths=[],
+    expected_models=[],
+    expected_groups={},
+    expected_skipped=0,
+    expected_metadata_count=0,
+    expected_duplicates=0,
+    expected_group_duplicates=0,
+    detailed=False,
+):
     expected_paths_str = ["    " + str(path) for path in expected_paths]
     expected_models_str = ["    " + name for name in expected_models]
-    expected_groups_str = ["    " + name + " [{} models]".format(len(models)) for name, models in expected_groups.items()]
+    expected_groups_str = [
+        "    " + name + " [{} models]".format(len(models)) for name, models in expected_groups.items()
+    ]
     skipped_count = 0
     duplicates_count = 0
     metadata_count = 0
@@ -97,14 +110,20 @@ def _check_summary_output(out, expected_paths=[], expected_models=[], expected_g
         if match:
             count = int(match.group(1))
             duplicates_count = duplicates_count + count
-            line = line[:line.find(" (")]
+            line = line[: line.find(" (")]
 
         if detailed:
             if line.startswith("        "):
                 if "Metadata: available" in line:
                     metadata_count = metadata_count + 1
                 continue
-        assert line in ["Models Summary", "Paths:", "Models:", "Groups:", ""] + expected_paths_str + expected_models_str + expected_groups_str
+        assert (
+            line
+            in ["Models Summary", "Paths:", "Models:", "Groups:", ""]
+            + expected_paths_str
+            + expected_models_str
+            + expected_groups_str
+        )
     assert skipped_count == expected_skipped
     assert duplicates_count == expected_duplicates + expected_group_duplicates
     if detailed:
@@ -114,7 +133,7 @@ def _check_summary_output(out, expected_paths=[], expected_models=[], expected_g
 @pytest.mark.parametrize("detailed", [False, True])
 def test_models_print_summary(detailed, capsys, fake_context, fake_environment_directory, fake_config_home):
     # list empty
-    with mock.patch.dict(fake_context.environment.paths, {"models": []}) as mocked:
+    with mock.patch.dict(fake_context.environment.paths, {"models": []}):
         print_summary(fake_context, detailed=detailed)
         out, err = capsys.readouterr()
         for line in out.split("\n"):
@@ -124,77 +143,84 @@ def test_models_print_summary(detailed, capsys, fake_context, fake_environment_d
     env_models_dir.mkdir()
 
     # single path, no models
-    with mock.patch.dict(fake_context.environment.paths, {"models": [PathConfig(env_models_dir)]}) as mocked:
+    with mock.patch.dict(fake_context.environment.paths, {"models": [PathConfig(env_models_dir)]}):
         print_summary(fake_context, detailed=detailed)
         out, err = capsys.readouterr()
         _check_summary_output(
-                out,
-                detailed=detailed,
-                expected_paths=[env_models_dir],
+            out,
+            detailed=detailed,
+            expected_paths=[env_models_dir],
         )
 
     _create_fake_models(env_models_dir, ["model0"], with_metadata=True)
     _create_fake_models(env_models_dir, ["model1", "model2"])
 
     # single path, 3 models
-    with mock.patch.dict(fake_context.environment.paths, {"models": [PathConfig(env_models_dir)]}) as mocked:
+    with mock.patch.dict(fake_context.environment.paths, {"models": [PathConfig(env_models_dir)]}):
         print_summary(fake_context, detailed=detailed)
         out, err = capsys.readouterr()
         _check_summary_output(
-                out,
-                detailed=detailed,
-                expected_paths=[env_models_dir],
-                expected_models=["model0", "model1", "model2"],
-                expected_metadata_count=1,
+            out,
+            detailed=detailed,
+            expected_paths=[env_models_dir],
+            expected_models=["model0", "model1", "model2"],
+            expected_metadata_count=1,
         )
 
-
     user_models_dir = fake_config_home / "models"
 
     # two paths (1 skipped), 3 models
-    with mock.patch.dict(fake_context.environment.paths, {"models": [PathConfig(env_models_dir), PathConfig(user_models_dir)]}) as mocked:
+    with mock.patch.dict(
+        fake_context.environment.paths, {"models": [PathConfig(env_models_dir), PathConfig(user_models_dir)]}
+    ):
         print_summary(fake_context, detailed=detailed)
         out, err = capsys.readouterr()
         _check_summary_output(
-                out,
-                detailed=detailed,
-                expected_paths=[env_models_dir, user_models_dir],
-                expected_models=["model0", "model1", "model2"],
-                expected_skipped=1,
-                expected_metadata_count=1,
+            out,
+            detailed=detailed,
+            expected_paths=[env_models_dir, user_models_dir],
+            expected_models=["model0", "model1", "model2"],
+            expected_skipped=1,
+            expected_metadata_count=1,
         )
 
     user_models_dir.mkdir()
     _create_fake_models(fake_config_home / "models", ["model2", "model3"])
 
     # two paths, 5 models, 1 duplicate
-    with mock.patch.dict(fake_context.environment.paths, {"models": [PathConfig(fake_environment_directory / "models"), PathConfig(fake_config_home / "models")]}) as mocked:
+    with mock.patch.dict(
+        fake_context.environment.paths,
+        {"models": [PathConfig(fake_environment_directory / "models"), PathConfig(fake_config_home / "models")]},
+    ):
         print_summary(fake_context, detailed=detailed)
         out, err = capsys.readouterr()
         _check_summary_output(
-                out,
-                detailed=detailed,
-                expected_paths=[env_models_dir, user_models_dir],
-                expected_models=["model0", "model1", "model2", "model3"],
-                expected_metadata_count=1,
-                expected_duplicates=1,
+            out,
+            detailed=detailed,
+            expected_paths=[env_models_dir, user_models_dir],
+            expected_models=["model0", "model1", "model2", "model3"],
+            expected_metadata_count=1,
+            expected_duplicates=1,
         )
 
     _create_fake_modelgroups(env_models_dir, {"mygroup": ["model0", "model2"]})
 
     # two paths, 5 models, 1 duplicate, 1 group
-    with mock.patch.dict(fake_context.environment.paths, {"models": [PathConfig(fake_environment_directory / "models"), PathConfig(fake_config_home / "models")]}) as mocked:
+    with mock.patch.dict(
+        fake_context.environment.paths,
+        {"models": [PathConfig(fake_environment_directory / "models"), PathConfig(fake_config_home / "models")]},
+    ):
         print_summary(fake_context, detailed=detailed)
         out, err = capsys.readouterr()
         print("OUT", out)
         _check_summary_output(
-                out,
-                detailed=detailed,
-                expected_paths=[env_models_dir, user_models_dir],
-                expected_models=["model0", "model1", "model2", "model3"],
-                expected_groups={"mygroup": ["model0", "model2"]},
-                expected_metadata_count=1,
-                expected_duplicates=1,
+            out,
+            detailed=detailed,
+            expected_paths=[env_models_dir, user_models_dir],
+            expected_models=["model0", "model1", "model2", "model3"],
+            expected_groups={"mygroup": ["model0", "model2"]},
+            expected_metadata_count=1,
+            expected_duplicates=1,
         )
     # TODO: group name conflicts with modelname
     # TODO: duplicate groups
