@@ -23,6 +23,7 @@ from mlonmcu.setup.setup import Setup
 
 TestTaskFactory = TaskFactory()
 
+
 def _validate_example_task1(context, params={}):
     print("_validate_example_task1", params)
     assert "foo" in params and "bar" in params and "special" in params
@@ -30,6 +31,7 @@ def _validate_example_task1(context, params={}):
         return params["special"]
     return False
 
+
 @TestTaskFactory.provides(["dep0"])
 @TestTaskFactory.param("special", [True, False])
 @TestTaskFactory.param("foo", [0, 1])
@@ -40,6 +42,7 @@ def example_task1(context, params={}, rebuild=False):
     context.cache._vars["dep0"] = ""
     pass
 
+
 @TestTaskFactory.needs(["dep0"])
 @TestTaskFactory.provides(["dep1"])
 @TestTaskFactory.register(category=TaskType.MISC)
@@ -47,6 +50,7 @@ def example_task2(context, params={}, rebuild=False):
     context.cache._vars["dep1"] = ""
     pass
 
+
 def test_task_registry():
     assert len(TestTaskFactory.registry) == 2
     assert len(TestTaskFactory.validates) == 1
@@ -56,11 +60,13 @@ def test_task_registry():
     assert len(TestTaskFactory.params) == 2
     assert len(TestTaskFactory.params["example_task1"]) == 3
 
+
 def test_task_registry_reset_changes():
     TestTaskFactory.changed = ["dep0"]
     TestTaskFactory.reset_changes()
     assert len(TestTaskFactory.changed) == 0
 
+
 @pytest.mark.parametrize("progress", [False, True])
 @pytest.mark.parametrize("print_output", [False, True])
 @pytest.mark.parametrize("rebuild", [False, True])  # TODO: actually test this
@@ -73,18 +79,21 @@ def test_setup_install_dependencies(progress, print_output, rebuild, write_cache
     config = {"print_output": print_output}
     installer = Setup(config=config, context=fake_context, tasks_factory=TestTaskFactory)
     result = installer.install_dependencies(progress=progress, write_cache=write_cache, rebuild=rebuild)
+    assert result
     # assert example_task1_mock.call_count == 3
-    assert TestTaskFactory.registry["example_task1"].call_count == 1  # Due to the mock, the actual wrapper is not executed anymore, params are not considered etc
+    assert (
+        TestTaskFactory.registry["example_task1"].call_count == 1
+    )  # Due to the mock, the actual wrapper is not executed anymore, params are not considered etc
     # assert example_task2_mock.call_count == 1
     assert TestTaskFactory.registry["example_task2"].call_count == 1
 
+
 def test_task_get_combs():
     assert get_combs({}) == []
     assert get_combs({"foo": []}) == []  # TODO: invalid?
     assert get_combs({"foo": [0]}) == [{"foo": 0}]
     # assert set(get_combs({"foo": [0, 1, 2]})) == set([{"foo": 0}, {"foo": 1}, {"foo": 2}])
     assert get_combs({"foo": [0, 1, 2]}) == [{"foo": 0}, {"foo": 1}, {"foo": 2}]
-    # assert set(get_combs({"foo": [0, 1], "bar": ["A", "B"]})) == set([{"foo": 0, "bar": "A"}, {"foo": 1, "bar": "A"}, {"foo": 0, "bar": "B"}, {"foo": 1, "bar": "B"}])  # order irrelevant
     assert len(get_combs({"foo": [0, 1], "bar": ["A", "B"]})) == 4
     assert len(get_combs({"foo": [0, 1], "bar": ["A", "B"]})[0].items()) == 2
 
@@ -103,4 +112,3 @@ def test_task_graph():
     assert len(order) == len(nodes)
     assert order.index("NodeB") > order.index("NodeA") and order.index("NodeB") > order.index("NodeC")
     assert order.index("NodeC") > order.index("NodeA")
-
